@@ -71,7 +71,6 @@
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
-#include <time.h>
 
 #define NNTPPORT 119
 
@@ -190,19 +189,19 @@ ptime(void)
   diff = elapsed;
   elapsed = do_time(start);
   diff = elapsed - diff;
-  printf("%5.1f %5.1f  ", ((double)elapsed)/1000.0, ((double)diff)/1000.0);
+  printf("%5.1f %5.1f  ",((float)elapsed)/1000.0,((float)diff)/1000.0);
 }
 
 void
-massagebuff(int bread, char *buffer)
+massagebuff(int bread, char *buf)
 {
   char *p;
 
   if ( bread > 55 )
-    strcpy(buffer+55," [...]\n");
+    strcpy(buf+55," [...]\n");
   else
-    buffer[bread] = '\0';
-  for ( p = buffer; *p != '\0'; )
+    buf[bread] = '\0';
+  for ( p = buf; *p != '\0'; )
     if ( *p != '\r' )  /* We like to do it RISC style. */
       p++;
     else {
