@@ -1,4 +1,4 @@
-/*  $Id: expireover.c 8418 2009-04-12 21:08:22Z iulius $
+/*  $Id: expireover.c 8574 2009-08-18 13:51:41Z iulius $
 **
 **  Expire the overview database.
 **
@@ -25,7 +25,7 @@
 #include "inn/storage.h"
 
 static const char usage[] = "\
-Usage: expireover [-ekNpqs] [-w offset] [-z rmfile] [-Z lowmarkfile]\n";
+Usage: expireover [-ekNpqs] [-f file] [-w offset] [-z rmfile] [-Z lowmarkfile]\n";
 
 /* Set to 1 if we've received a signal; expireover then terminates after
    finishing the newsgroup that it's working on (this prevents corruption of
