@@ -1,8 +1,8 @@
-/*  $Id: fastrm.c 8225 2008-12-14 05:05:09Z eagle $
+/*  $Id: fastrm.c 9845 2015-05-02 16:32:08Z iulius $
 **
 **  Delete a list of filenames or tokens from stdin.
 **
-**  Originally written by <kre@munnari.oz.au> (to only handle files)
+**  Originally written by <kre@munnari.oz.au> (to only handle files).
 **
 **  Files that can't be unlinked because they didn't exist are considered
 **  okay.  Any error condition results in exiting with non-zero exit
@@ -481,7 +481,7 @@ setup_dir(char *dir, int filecount)
 
        FIXME: It's too hard to figure out what this code does.  It needs to be
        rewritten. */
-    if (p != '\0' && relative_threshold > 0) {
+    if (*p != '\0' && relative_threshold > 0) {
         depth = slashcount(q);
         if (depth <= relative_threshold) {
             while (p > absolute && *--p != '/')
@@ -666,7 +666,7 @@ main(int argc, char *argv[])
                 continue;
             case 'c':
                 chdir_threshold = 1;
-                if (!CTYPE(isdigit, p[1]))
+                if (!isdigit((unsigned char) p[1]))
                     continue;
                 chdir_threshold = atoi(p + 1);
                 break;
@@ -678,13 +678,13 @@ main(int argc, char *argv[])
                 continue;
             case 's':
                 sort_threshold = 5;
-                if (!CTYPE(isdigit, p[1]))
+                if (!isdigit((unsigned char) p[1]))
                     continue;
                 sort_threshold = atoi(p + 1);
                 break;
             case 'u':
                 relative_threshold = 1;
-                if (!CTYPE(isdigit, p[1]))
+                if (!isdigit((unsigned char) p[1]))
                     continue;
                 relative_threshold = atoi(p + 1);
                 if (relative_threshold >= (int) strlen(dotdots) / 3)
