@@ -1,4 +1,4 @@
-/*  $Id: rnews.c,v 1.54.2.4 2004/06/03 23:52:56 rra Exp $
+/*  $Id: rnews.c 9855 2015-05-14 13:13:11Z iulius $
 **
 **  A front-end for InterNetNews.
 **
@@ -14,17 +14,17 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <pwd.h>
 #include <syslog.h>
 #include <sys/stat.h>
 
 #include "inn/innconf.h"
+#include "inn/libinn.h"
 #include "inn/messages.h"
+#include "inn/newsuser.h"
+#include "inn/nntp.h"
+#include "inn/paths.h"
+#include "inn/storage.h"
 #include "inn/wire.h"
-#include "libinn.h"
-#include "nntp.h"
-#include "paths.h"
-#include "storage.h"
 
 
 typedef struct _HEADER {
@@ -58,7 +58,7 @@ static HEADER	RequiredHeaders[] = {
 #define IS_MESGID(hp)	((hp) == &RequiredHeaders[_messageid])
 #define IS_PATH(hp)	((hp) == &RequiredHeaders[_path])
 
-
+
 
 /*
 **  Open up a pipe to a process with fd tied to its stdin.  Return a
@@ -77,7 +77,7 @@ StartChild(int fd, const char *path, const char *argv[])
 
     /* Get a child. */
     for (i = 0; (pid = fork()) < 0; i++) {
-	if (i == innconf->maxforks) {
+	if (i == (long) innconf->maxforks) {
             syswarn("cannot fork %s, spooling", path);
 	    return -1;
 	}
@@ -137,7 +137,7 @@ WaitForChildren(int n)
 }
 
 
-
+
 
 /*
 **  Clean up the NNTP escapes from a line.
@@ -159,13 +159,14 @@ static char *REMclean(char *buff)
 /*
 **  Write an article to the rejected directory.
 */
-static void Reject(const char *article, const char *reason, const char *arg)
+static void
+Reject(const char *article, size_t length UNUSED, const char *reason,
+       const char *arg)
 {
 #if	defined(DO_RNEWS_SAVE_BAD)
     char *filename;
     FILE *F;
     int fd;
-    size_t length;
 #endif	/* defined(DO_RNEWS_SAVE_BAD) */
 
     notice(reason, arg);
@@ -187,7 +188,6 @@ static void Reject(const char *article, const char *reason, const char *arg)
         warn("cannot fdopen %s", filename);
 	return;
     }
-    length = strlen(article);
     if (fwrite(article, 1, length, F) != length)
         warn("cannot fwrite to %s", filename);
     if (fclose(F) == EOF)
@@ -202,7 +202,8 @@ static void Reject(const char *article, const char *reason, const char *arg)
 **  whole batch needs to be saved (such as when the server goes down or if
 **  the file is corrupted).
 */
-static bool Process(char *article)
+static bool
+Process(char *article, size_t artlen)
 {
     HEADER	        *hp;
     const char	        *p;
@@ -223,15 +224,15 @@ static bool Process(char *article)
 	return true;
 
     /* Convert the article to wire format. */
-    wirefmt = ToWireFmt(article, strlen(article), &length);
+    wirefmt = wire_from_native(article, artlen, &length);
 
     /* Make sure that all the headers are there, note the ID. */
     for (hp = RequiredHeaders; hp < ARRAY_END(RequiredHeaders); hp++) {
-        p = wire_findheader(wirefmt, length, hp->Name);
+        p = wire_findheader(wirefmt, length, hp->Name, true);
         if (p == NULL) {
             free(wirefmt);
-	    Reject(article, "bad_article missing %s", hp->Name);
-	    return false;
+	    Reject(article, artlen, "bad_article missing %s", hp->Name);
+	    return true;
 	}
 	if (IS_MESGID(hp)) {
 	    id = p;
@@ -249,8 +250,9 @@ static bool Process(char *article)
     /* Send the NNTP "ihave" message. */
     if ((p = strchr(id, '\r')) == NULL) {
         free(wirefmt);
-	Reject(article, "bad_article unterminated %s header", "Message-ID");
-	return false;
+	Reject(article, artlen, "bad_article unterminated %s header",
+               "Message-ID");
+	return true;
     }
     msgid = xstrndup(id, p - id);
     fprintf(ToServer, "ihave %s\r\n", msgid);
@@ -269,7 +271,7 @@ static bool Process(char *article)
 	return false;
     }
     REMclean(buff);
-    if (!CTYPE(isdigit, buff[0])) {
+    if (!isdigit((unsigned char) buff[0])) {
         free(wirefmt);
         notice("bad_reply after ihave %s", buff);
 	return false;
@@ -277,20 +279,20 @@ static bool Process(char *article)
     switch (atoi(buff)) {
     default:
         free(wirefmt);
-	Reject(article, "unknown_reply after ihave %s", buff);
-	return true;
-    case NNTP_RESENDIT_VAL:
+        notice("unknown_reply after ihave %s", buff);
+	return false;
+    case NNTP_FAIL_IHAVE_DEFER:
         free(wirefmt);
 	return false;
-    case NNTP_SENDIT_VAL:
+    case NNTP_CONT_IHAVE:
 	break;
-    case NNTP_HAVEIT_VAL:
+    case NNTP_FAIL_IHAVE_REFUSE:
 #if	defined(SYSLOG_RNEWS_LOG_DUPS)
 	*p = '\0';
         notice("duplicate %s %s", id, path);
 #endif	/* defined(SYSLOG_RNEWS_LOG_DUPS) */
 #if	defined(FILE_RNEWS_LOG_DUPS)
-	if ((F = fopen(_PATH_RNEWS_DUP_LOG, "a")) != NULL) {
+	if ((F = fopen(INN_PATH_RNEWS_DUP_LOG, "a")) != NULL) {
 	    *p = '\0';
 	    fprintf(F, "duplicate %s %s\n", id, path);
 	    fclose(F);
@@ -323,7 +325,7 @@ static bool Process(char *article)
 	return false;
     }
     REMclean(buff);
-    if (!CTYPE(isdigit, buff[0])) {
+    if (!isdigit((unsigned char) buff[0])) {
         notice("bad_reply after article %s", buff);
 	return false;
     }
@@ -331,12 +333,12 @@ static bool Process(char *article)
     default:
         notice("unknown_reply after article %s", buff);
 	/* FALLTHROUGH */
-    case NNTP_RESENDIT_VAL:
+    case NNTP_FAIL_IHAVE_DEFER:
 	return false;
-    case NNTP_TOOKIT_VAL:
+    case NNTP_OK_IHAVE:
 	break;
-    case NNTP_REJECTIT_VAL:
-	Reject(article, "rejected %s", buff);
+    case NNTP_FAIL_IHAVE_REJECT:
+	Reject(article, artlen, "rejected %s", buff);
 	break;
     }
     return true;
@@ -398,7 +400,7 @@ ReadRemainder(int fd, char first, char second)
 	article[used++] = '\n';
     article[used] = '\0';
 
-    ok = Process(article);
+    ok = Process(article, used);
     free(article);
     return ok;
 }
@@ -427,28 +429,27 @@ ReadBytecount(int fd, int artsize)
         article = xrealloc(article, oldsize + 1 + 1);
     }
 
-    /* Read in the article. */
+    /* Read in the article.  If we don't get as many bytes as we should,
+       return true without doing anything, throwing away the article.  This
+       seems like the best of a set of bad options; Reject would save the
+       article into bad and then someone might reprocess it, leaving us with
+       accepting the truncated version. */
     for (p = article, left = artsize; left; p += i, left -= i)
 	if ((i = read(fd, p, left)) <= 0) {
 	    i = errno;
             warn("cannot read, wanted %d got %d", artsize, artsize - left);
-#if	0
-	    /* Don't do this -- if the article gets re-processed we
-	     * will end up accepting the truncated version. */
-	    artsize = p - article;
-	    article[artsize] = '\0';
-	    Reject(article, "short read (%s?)", strerror(i));
-#endif	/* 0 */
 	    return true;
 	}
-    if (p[-1] != '\n')
+    if (p[-1] != '\n') {
 	*p++ = '\n';
+        artsize++;
+    }
     *p = '\0';
 
-    return Process(article);
+    return Process(article, artsize);
 }
 
-
+
 
 /*
 **  Read a single text line; not unlike fgets().  Just more inefficient.
@@ -527,7 +528,7 @@ UnpackOne(int *fdp, size_t *countp)
 	    cargv[1] = "-d";
 	    cargv[2] = NULL;
 	    lseek(*fdp, 0, 0); /* Back to the beginning */
-	    *fdp = StartChild(*fdp, _PATH_GZIP, cargv);
+	    *fdp = StartChild(*fdp, INN_PATH_GZIP, cargv);
 	    if (*fdp < 0)
 	        return false;
 	    (*countp)++;
@@ -565,7 +566,7 @@ UnpackOne(int *fdp, size_t *countp)
 	    cargv[0] = UNPACK;
 	    cargv[1] = "-d";
 	    cargv[2] = NULL;
-	    *fdp = StartChild(*fdp, _PATH_GZIP, cargv);
+	    *fdp = StartChild(*fdp, INN_PATH_GZIP, cargv);
 	    if (*fdp < 0)
 		return false;
 	    (*countp)++;
@@ -581,13 +582,13 @@ UnpackOne(int *fdp, size_t *countp)
 	    p++;
 	else
 	    p = &buff[3];
-	if (strchr(_PATH_RNEWSPROGS, '/') == NULL) {
+	if (strchr(INN_PATH_RNEWSPROGS, '/') == NULL) {
 	    snprintf(path, sizeof(path), "%s/%s/%s", innconf->pathbin,
-                     _PATH_RNEWSPROGS, p);
-	    len = strlen(innconf->pathbin) + 1 + sizeof _PATH_RNEWSPROGS;
+                     INN_PATH_RNEWSPROGS, p);
+	    len = strlen(innconf->pathbin) + 1 + sizeof INN_PATH_RNEWSPROGS;
 	} else {
-	    snprintf(path, sizeof(path), "%s/%s", _PATH_RNEWSPROGS, p);
-	    len = sizeof _PATH_RNEWSPROGS;
+	    snprintf(path, sizeof(path), "%s/%s", INN_PATH_RNEWSPROGS, p);
+	    len = sizeof INN_PATH_RNEWSPROGS;
 	}
 	for (p = &path[len]; *p; p++)
 	    if (ISWHITE(*p)) {
@@ -620,10 +621,10 @@ Unspool(void)
     struct dirent       *ep;
     bool	ok;
     struct stat		Sb;
-    char		hostname[10];
-    int			fd, fd2;
+    char		hostname[SMBUF];
+    int			fd;
     size_t		i;
-    char                *badname, *uuhost;
+    char                *uuhost;
 
     message_handlers_die(2, message_log_stderr, message_log_syslog_err);
     message_handlers_warn(2, message_log_stderr, message_log_syslog_err);
@@ -674,22 +675,15 @@ Unspool(void)
 	WaitForChildren(i);
 	UUCPHost = uuhost;
 
-	if (!ok) {
-            badname = concat(PathBadNews, "/", hostname, "XXXXXX", (char *) 0);
-            fd2 = mkstemp(badname);
-            if (fd2 < 0)
-                sysdie("cannot create temporary file");
-            close(fd2);
-            warn("cant unspool saving to %s", badname);
-	    if (rename(InputFile, badname) < 0)
-                sysdie("cannot rename %s to %s", InputFile, badname);
-	    close(fd);
-            free(badname);
-	    continue;
-	}
+        /* If UnpackOne returned true, the article has been dealt with one way
+           or the other, so remove it.  Otherwise, leave it in place; either
+           we got an unknown error from the server or we got a deferral, and
+           for both we want to try later. */
+	if (ok) {
+            if (unlink(InputFile) < 0)
+                syswarn("cannot remove %s", InputFile);
+        }
 
-	if (unlink(InputFile) < 0)
-            syswarn("cannot remove %s", InputFile);
 	close(fd);
     }
     closedir(dp);
@@ -698,7 +692,7 @@ Unspool(void)
     message_handlers_warn(1, message_log_syslog_err);
 }
 
-
+
 
 /*
 **  Can't connect to the server, so spool our input.  There isn't much
@@ -774,15 +768,15 @@ Spool(int fd, int mode)
 **  Try to read the password file and open a connection to a remote
 **  NNTP server.
 */
-static bool OpenRemote(char *server, int port, char *buff)
+static bool OpenRemote(char *server, int port, char *buff, size_t len)
 {
     int		i;
 
     /* Open the remote connection. */
     if (server)
-	i = NNTPconnect(server, port, &FromServer, &ToServer, buff);
+	i = NNTPconnect(server, port, &FromServer, &ToServer, buff, len);
     else
-	i = NNTPremoteopen(port, &FromServer, &ToServer, buff);
+	i = NNTPremoteopen(port, &FromServer, &ToServer, buff, len);
     if (i < 0)
 	return false;
 
@@ -830,25 +824,44 @@ int main(int ac, char *av[])
     message_handlers_warn(1, message_log_syslog_err);
     message_handlers_die(1, message_log_syslog_err);
 
+    /* The reason for the following is somewhat obscure and is done only
+       because rnews is sometimes installed setuid.
+
+       The stderr stream used by message_log_syslog_err is associated with
+       file descriptor 2, generally even if that file descriptor is closed.
+       Someone running rnews may close all of the standard file descriptors
+       before running it, in which case, later in its operations, one of the
+       article files or network connections it has open could be file
+       descriptor 2.  If an error occurs at that point, the error message may
+       be written to that file or network connection instead of to stderr,
+       with unpredictable results.
+
+       We avoid this by burning three file descriptors if the real and
+       effective user IDs don't match, or if we're running as root.  (If they
+       do match, there is no escalation of privileges and at worst the user is
+       just managing to produce a strange bug.) */
+    if (getuid() != geteuid() || geteuid() == 0) {
+        if (open("/dev/null", O_RDONLY) < 0)
+            sysdie("cannot open /dev/null");
+        if (open("/dev/null", O_RDONLY) < 0)
+            sysdie("cannot open /dev/null");
+        if (open("/dev/null", O_RDONLY) < 0)
+            sysdie("cannot open /dev/null");
+    }
+
     /* Make sure that we switch to the news user if we're running as root,
        since we may spool files and don't want those files owned by root.
        Don't require that we be running as the news user, though; there are
        other setups where rnews might be setuid news or be run by other
        processes in the news group. */
     if (getuid() == 0 || geteuid() == 0) {
-        struct passwd *pwd;
-
-        pwd = getpwnam(NEWSUSER);
-        if (pwd == NULL)
-            die("can't resolve %s to a UID (account doesn't exist?)",
-                NEWSUSER);
-        setuid(pwd->pw_uid);
+        ensure_news_user(true);
     }
 
     if (!innconf_read(NULL))
         exit(1);
-    UUCPHost = getenv(_ENV_UUCPHOST);
-    PathBadNews = concatpath(innconf->pathincoming, _PATH_BADNEWS);
+    UUCPHost = getenv(INN_ENV_UUCPHOST);
+    PathBadNews = concatpath(innconf->pathincoming, INN_PATH_BADNEWS);
     port = innconf->nnrpdpostport;
 
     umask(NEWSUMASK);
@@ -901,28 +914,23 @@ int main(int ac, char *av[])
 
     /* Open the link to the server. */
     if (remoteServer != NULL) {
-	if (!OpenRemote(remoteServer,port,buff))
+	if (!OpenRemote(remoteServer, port, buff, sizeof(buff)))
 		CantConnect(buff,mode,fd);
     } else if (innconf->nnrpdposthost != NULL) {
 	if (!OpenRemote(innconf->nnrpdposthost,
-	    (port != NNTP_PORT) ? port : innconf->nnrpdpostport, buff))
+                        (port != NNTP_PORT) ? (unsigned) port : innconf->nnrpdpostport,
+                        buff, sizeof(buff)))
 		CantConnect(buff, mode, fd);
     }
     else {
-#if	defined(DO_RNEWSLOCALCONNECT)
-	if (NNTPlocalopen(&FromServer, &ToServer, buff) < 0) {
+	if (NNTPlocalopen(&FromServer, &ToServer, buff, sizeof(buff)) < 0) {
 	    /* If server rejected us, no point in continuing. */
 	    if (buff[0])
 		CantConnect(buff, mode, fd);
-	    if (!OpenRemote((char *)NULL,
-	    	(port != NNTP_PORT) ? port : innconf->port, buff))
+	    if (!OpenRemote(NULL, (port != NNTP_PORT) ? (unsigned) port : innconf->port,
+                            buff, sizeof(buff)))
 			CantConnect(buff, mode, fd);
 	}
-#else
-	if (!OpenRemote((char *)NULL, 
-	    (port != NNTP_PORT) ? port : innconf->port, buff))
-		CantConnect(buff, mode, fd);
-#endif	/* defined(DO_RNEWSLOCALCONNECT) */
     }
     close_on_exec(fileno(FromServer), true);
     close_on_exec(fileno(ToServer), true);
