@@ -1,4 +1,4 @@
-/*  $Id: buffer.h 7011 2004-10-01 22:10:01Z rra $
+/*  $Id: buffer.h 8838 2009-12-03 21:11:00Z iulius $
 **
 **  Counted, reusable memory buffer.
 **
@@ -52,7 +52,8 @@ void buffer_append(struct buffer *, const char *data, size_t length);
 
 /* Print data into the buffer, either appending or replacing the existing
    data.  No trailing nul is added. */
-void buffer_sprintf(struct buffer *, bool append, const char *, ...);
+void buffer_sprintf(struct buffer *, bool append, const char *, ...)
+    __attribute__((__format__(printf, 3, 4)));
 void buffer_vsprintf(struct buffer *, bool append, const char *, va_list);
 
 /* Swap the contents of two buffers. */
