@@ -3,7 +3,6 @@
 
 /* Need the definition of HASH. */
 #include "inn/libinn.h"
-#include "inn/portable-macros.h"
 
 BEGIN_DECLS
 
@@ -39,17 +38,20 @@ typedef struct {
     bool             nonblock;
 } dbzoptions;
 
+#ifdef __GNUC__
+#define PACKED __attribute__ ((packed))
+#else
+#if !defined(PACKED)
+#define PACKED
+#endif
+#endif
+
 #if !defined(lint) && (defined(__SUNPRO_C) || defined(_nec_ews))
 #pragma pack(1)
 #endif /* nor lint, nor __SUNPRO_C, nor sgi, nor _nec_ews */
-/* Leave the __attribute__ ((__packed__)) on there because removing it
- * might change the layout of the data structure on disk on some platform,
- * thus invalidating old history files.
- * It is fairly unlikely that this is a problem, though.
- */
 typedef struct {
     char		hash[DBZ_INTERNAL_HASH_SIZE];
-} __attribute__ ((__packed__)) erec;
+} PACKED erec;
 #if !defined(lint) && (defined(__SUNPRO_C) || defined(_nec_ews))
 #pragma pack()
 #endif /* nor lint, nor__SUNPRO_C, nor _nec_ews */
@@ -69,11 +71,6 @@ extern long dbzsize(off_t contents);
 extern void dbzsetoptions(const dbzoptions options);
 extern void dbzgetoptions(dbzoptions *options);
 
-#ifdef DBZTEST
-extern int timediffms(struct timeval start, struct timeval end);
-extern void RemoveDBZ(char *filename);
-#endif /* DBZTEST */
-
 END_DECLS
 
 #endif /* INN_DBZ_H */
