@@ -1,4 +1,4 @@
-/*  $Id: dispatch.h 9782 2015-01-07 21:34:22Z iulius $
+/*  $Id: dispatch.h 9780 2015-01-07 21:03:00Z iulius $
 **
 **  Dispatch cvectors of commands to functions.
 **
@@ -10,7 +10,7 @@
 #ifndef INN_DISPATCH_H
 #define INN_DISPATCH_H 1
 
-#include "inn/portable-macros.h"
+#include <inn/defines.h>
 
 /* Forward declarations. */
 struct cvector;
