@@ -1,4 +1,4 @@
-/*  $Id: innconf.h 8927 2010-01-22 23:39:07Z iulius $
+/*  $Id: innconf.h 9753 2014-12-01 20:26:38Z iulius $
 **
 **  inn.conf parser interface.
 **
@@ -127,6 +127,11 @@ struct innconf {
     char *tlscapath;            /* Path to a directory of CA certificates */
     char *tlscertfile;          /* Path to the SSL certificate to use */
     char *tlskeyfile;           /* Path to the key for the certificate */
+    char *tlsciphers;           /* OpenSSL-style cipher string */
+    bool tlscompression;        /* Turn TLS compression on/off */
+    char *tlseccurve;           /* ECDH curve name */
+    bool tlspreferserverciphers; /* Make server select the cipher */
+    struct vector *tlsprotocols; /* List of supported TLS versions */
 #endif /* HAVE_SSL */
 
     /* Monitoring */
@@ -142,6 +147,7 @@ struct innconf {
 
     /* Logging */
     bool docnfsstat;            /* Run cnfsstat in the background? */
+    bool htmlstatus;            /* Write status file as HTML instead of plain text? */
     unsigned long incominglogfrequency;  /* Checkpoint log after this many articles */
     bool logartsize;            /* Log article sizes? */
     bool logcancelcomm;         /* Log ctlinnd cancel commands to syslog? */
