@@ -1,12 +1,14 @@
+/*  $Id: ov.h 9780 2015-01-07 21:03:00Z iulius $
+*/
+
 #ifndef INN_OV_H
 #define INN_OV_H 1
 
+#include <inn/defines.h>
 #include "storage.h"
 #include "inn/history.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+BEGIN_DECLS
 
 #define OV_READ  1
 #define OV_WRITE 2
@@ -75,8 +77,6 @@ enum {
 };
 #endif
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+END_DECLS
 
-#endif /* !INN_OV_H */
+#endif /* INN_OV_H */
