@@ -1,4 +1,4 @@
-/*  $Id: overview.h 8409 2009-04-11 22:13:41Z iulius $
+/*  $Id: overview.h 9780 2015-01-07 21:03:00Z iulius $
 **
 **  Overview API for storing and retrieving overview information.
 **
@@ -11,6 +11,7 @@
 #ifndef INN_OVERVIEW_H
 #define INN_OVERVIEW_H 1
 
+#include <inn/defines.h>
 #include <stdio.h>              /* FILE */
 
 #include "inn/storage.h"
@@ -184,8 +185,8 @@ bool overview_check(const char *data, size_t length, ARTNUM article);
 int overview_index(const char *field, const struct vector *extra);
 struct cvector *overview_split(const char *line, size_t length,
 			       ARTNUM *number, struct cvector *vector);
-char *overview_getheader(const struct cvector *vector, unsigned int element,
-			 const struct vector *extra);
+char *overview_get_standard_header(const struct cvector *vector, unsigned int element);
+char *overview_get_extra_header(const struct cvector *vector, const char *header);
 
 END_DECLS
 
