@@ -1,4 +1,4 @@
-/*  $Id: mmap.h 6012 2002-12-16 11:19:21Z alexk $
+/*  $Id: mmap.h 9719 2014-09-22 19:19:40Z iulius $
 **
 **  Portability wrapper around <sys/mman.h>.
 **
@@ -21,9 +21,10 @@
 
 /* Solaris 8 (at least) prototypes munmap, msync, and madvise as taking char *
    (actually a caddr_t, which is a typedef for a char *) instead of void * as
-   is required by the standard.  This macro adds a cast that silences compiler
+   is required by the standard.  These macros add casts that silences compiler
    warnings on Solaris 8 without adversely affecting other platforms.  (ISO C
-   allows macro definitions of this sort; this macro is not recursive.) */
+   allows macro definitions of this sort; this macro is not recursive.)
+   Do not redefine mmap because at least AIX 7.1 redefines it to mmap64. */
 #define munmap(p, l)            munmap((void *)(p), (l))
 
 /* On some platforms, msync only takes two arguments.  (ANSI C allows macro
