@@ -1,4 +1,4 @@
-/*  $Id: art.c 9521 2013-07-31 17:27:33Z iulius $
+/*  $Id: art.c 9836 2015-05-02 11:57:40Z iulius $
 **
 **  Article-processing.
 */
@@ -1688,7 +1688,7 @@ ARTpropagate(ARTDATA *data, const char **hops, int hopcount, char **list,
       } else if (HDR_FOUND(HDR__INJECTION_INFO)) {
         begin = (char *) skip_cfws(HDR(HDR__INJECTION_INFO));
 
-        if (begin == '\0')
+        if (*begin == '\0')
           continue;
 
         /* The path identity ends with ';' or CFWS. */
