@@ -1,4 +1,4 @@
-/*  $Id: chan.c 8557 2009-08-05 21:02:53Z iulius $
+/*  $Id: chan.c 9801 2015-03-21 13:37:54Z iulius $
 **
 **  I/O channel (and buffer) processing.
 **
@@ -188,12 +188,14 @@ CHANcreate(int fd, enum channel_type type, enum channel_state state,
     /* Set up the channel's info.  Note that we don't have to initialize
        anything that's already set properly to zero in CHANnull. */
     *cp = CHANnull;
+    cp->av = NULL;
     cp->fd = fd;
     cp->Type = type;
     cp->State = state;
     cp->Reader = reader;
     cp->WriteDone = write_done;
     cp->Started = Now.tv_sec;
+    cp->Started_checkpoint = Now.tv_sec;
     cp->LastActive = Now.tv_sec;
     cp->In = in;
     cp->Out = out;
@@ -247,7 +249,7 @@ CHANtracing(CHANNEL *cp, bool flag)
     notice("%s trace %s", name, flag ? "on" : "off");
     cp->Tracing = flag;
     if (flag) {
-        notice("%s trace badwrites %d blockwrites %d badreads %d", name,
+        notice("%s trace badwrites %lu blockwrites %lu badreads %lu", name,
                cp->BadWrites, cp->BlockedWrites, cp->BadReads);
         network_sockaddr_sprint(addr, sizeof(addr),
                                 (struct sockaddr *) &cp->Address);
@@ -285,6 +287,17 @@ CHANclose_nntp(CHANNEL *cp, const char *name)
         notice("%s closed seconds %ld cancels %ld", name,
                (long)(Now.tv_sec - cp->Started), cp->Received);
     else {
+        notice("%s checkpoint seconds %ld accepted %ld refused %ld rejected %ld"
+               " duplicate %ld accepted size %.0f duplicate size %.0f"
+               " rejected size %.0f", name,
+               (long)(Now.tv_sec - cp->Started_checkpoint),
+               cp->Received - cp->Received_checkpoint,
+               cp->Refused - cp->Refused_checkpoint,
+               cp->Rejected - cp->Rejected_checkpoint,
+               cp->Duplicate - cp->Duplicate_checkpoint,
+               cp->Size - cp->Size_checkpoint,
+               cp->DuplicateSize - cp->DuplicateSize_checkpoint,
+               cp->RejectSize - cp->RejectSize_checkpoint);
         notice("%s closed seconds %ld accepted %ld refused %ld rejected %ld"
                " duplicate %ld accepted size %.0f duplicate size %.0f"
                " rejected size %.0f", name, (long)(Now.tv_sec - cp->Started),
@@ -375,7 +388,7 @@ CHANclose(CHANNEL *cp, const char *name)
         if (cp->Type == CTnntp)
             CHANclose_nntp(cp, name);
         else if (cp->Type == CTreject)
-            notice("%s %ld", name, cp->Rejected);
+            notice("%s %ld", name, cp->Rejected); /* Use cp->Rejected for the response code. */
         else if (cp->Out.left)
             warn("%s closed lost %lu", name, (unsigned long) cp->Out.left);
         else
@@ -423,6 +436,13 @@ CHANclose(CHANNEL *cp, const char *name)
         free(cp->Sendid.data);
         cp->Sendid.data = NULL;
     }
+
+    /* Free the space allocated for NNTP commands. */
+    if (cp->av != NULL) {
+        free(cp->av[0]);
+        free(cp->av);
+        cp->av = NULL;
+    }
 }
 
 
@@ -550,6 +570,21 @@ CHANresetlast(int fd)
 }
 
 
+/*
+**  When removing a channel from the sleep mask, we want to lower the last
+**  file descriptor if we removed the highest one.  Called from SCHANremove.
+*/
+static void
+CHANresetlastsleeping(int fd)
+{
+    if (fd == channels.max_sleep_fd) {
+        while (   !FD_ISSET(channels.max_sleep_fd, &channels.sleep_set)
+               && channels.max_sleep_fd > 1)
+            channels.max_sleep_fd--;
+    }
+}
+
+
 /*
 **  Mark a channel as an active reader.
 */
@@ -609,8 +644,6 @@ SCHANadd(CHANNEL *cp, time_t wake, void *event, innd_callback_func waker,
 void
 SCHANremove(CHANNEL *cp)
 {
-    int fd;
-
     if (!CHANsleeping(cp))
         return;
     FD_CLR(cp->fd, &channels.sleep_set);
@@ -618,12 +651,7 @@ SCHANremove(CHANNEL *cp)
     cp->Waketime = 0;
 
     /* If this was the highest descriptor, get a new highest. */
-    if (cp->fd == channels.max_sleep_fd) {
-        fd = channels.max_sleep_fd;
-        while (!FD_ISSET(fd, &channels.sleep_set) && fd > 1)
-            fd--;
-        channels.max_sleep_fd = fd;
-    }
+    CHANresetlastsleeping(cp->fd);
 }
 
 
@@ -787,8 +815,8 @@ CHANreadtext(CHANNEL *cp)
 
        FIXME: A better approach would be to limit the number of commands we
        process for each channel. */
-    if (innconf->maxcmdreadsize <= 0 || cp->State != CSgetcmd
-        || bp->left < (unsigned long) innconf->maxcmdreadsize)
+    if (innconf->maxcmdreadsize == 0 || cp->State != CSgetcmd
+        || bp->left < innconf->maxcmdreadsize)
         maxbyte = bp->left;
     else
         maxbyte = innconf->maxcmdreadsize;
@@ -906,7 +934,7 @@ CHANwakeup(CHANNEL *cp)
 static void
 CHANwritesleep(CHANNEL *cp, const char *name)
 {
-    int bad, delay;
+    unsigned long bad, delay;
 
     bad = ++(cp->BlockedWrites);
     if (bad > innconf->badiocount)
@@ -924,7 +952,7 @@ CHANwritesleep(CHANNEL *cp, const char *name)
             return;
         }
     delay = bad * innconf->blockbackoff;
-    warn("%s blocked sleeping %d", name, delay);
+    warn("%s blocked sleeping %lu", name, delay);
     SCHANadd(cp, Now.tv_sec + delay, NULL, CHANwakeup, NULL);
 }
 
@@ -1116,7 +1144,7 @@ CHANreadloop(void)
     CHANNEL *cp;
     fd_set rdfds, wrfds;
     struct timeval tv;
-    long silence;
+    unsigned long silence;
     const char *name;
     time_t last_sync;
     int fd = 0;
@@ -1133,10 +1161,10 @@ CHANreadloop(void)
         rdfds = channels.read_set;
         wrfds = channels.write_set;
         tv = TimeOut;
-        if (innconf->timer) {
+        if (innconf->timer != 0) {
             unsigned long now = TMRnow();
 
-            if (now < 1000 * (unsigned long)(innconf->timer))
+            if (now < 1000 * innconf->timer)
                 tv.tv_sec = innconf->timer - now / 1000;
             else {
                 TMRsummary("ME", timer_name);
@@ -1148,26 +1176,27 @@ CHANreadloop(void)
         count = select(channels.max_fd + 1, &rdfds, &wrfds, NULL, &tv);
         TMRstop(TMR_IDLE);
 
-        STATUSmainloophook();
-        if (GotTerminate) {
-            warn("%s exiting due to signal", LogName);
-            CleanupAndExit(0, NULL);
-        }
         if (count < 0) {
             if (errno != EINTR) {
                 syswarn("%s cant select", LogName);
 #ifdef INND_FIND_BAD_FDS
                 CHANdiagnose();
-#endif
+#endif      
             }
             continue;
         }
 
+        STATUSmainloophook();
+        if (GotTerminate) {
+            warn("%s exiting due to signal", LogName);
+            CleanupAndExit(0, NULL);
+        }
+
         /* Update the "reasonably accurate" time. */
         gettimeofday(&Now, NULL);
         if (Now.tv_sec > last_sync + TimeOut.tv_sec) {
             HISsync(History);
-            if (ICDactivedirty) {
+            if (ICDactivedirty != 0) {
                 ICDwriteactive();
                 ICDactivedirty = 0;
             }
@@ -1255,12 +1284,21 @@ CHANreadloop(void)
                 if (cp->Type == CTfree) {
                     warn("%s %d free but was in SMASK", CHANname(cp), fd);
                     FD_CLR(fd, &channels.sleep_set);
+                    channels.sleep_count--;
+                    CHANresetlastsleeping(fd);
                     close(fd);
                     cp->fd = -1;
                 } else {
                     cp->LastActive = Now.tv_sec;
                     SCHANremove(cp);
-                    (*cp->Waker)(cp);
+                    if (cp->Waker != NULL) {
+                        (*cp->Waker)(cp);
+                    } else {
+                        name = CHANname(cp);
+                        warn("%s %d sleeping without Waker", name, fd);
+                        SITEchanclose(cp);
+                        CHANclose(cp, name);
+                    }
                 }
             }
 
