@@ -1,4 +1,4 @@
-/*  $Id: nc.c 9009 2010-03-16 19:52:12Z iulius $
+/*  $Id: nc.c 9399 2012-03-04 15:08:17Z iulius $
 **
 **  Routines for the NNTP channel.  Other channels get the descriptors which
 **  we turn into NNTP channels, and over which we speak NNTP.
@@ -72,7 +72,7 @@ static NCDISPATCH NCcommands[] = {
     COMMAND("IHAVE",         NCihave,         true,  2,  2, true,
             "message-ID"),
     COMMAND("LIST",          NClist,          true,  1,  3, true,
-            "[ACTIVE|ACTIVE.TIMES|NEWSGROUPS [wildmat]]"),
+            "[ACTIVE [wildmat]|ACTIVE.TIMES [wildmat]|MOTD|NEWSGROUPS [wildmat]]"),
     COMMAND("MODE",          NCmode,          false, 2,  2, true,
             "READER"),
     COMMAND("QUIT",          NCquit,          false, 1,  1, true,
@@ -151,7 +151,7 @@ NCwritereply(CHANNEL *cp, const char *text)
     int i;
 
     /* XXX could do RCHANremove(cp) here, as the old NCwritetext() used to
-     * do, but that would be wrong if the channel is sreaming (because it
+     * do, but that would be wrong if the channel is streaming (because it
      * would zap the channel's input buffer).  There's no harm in
      * never calling RCHANremove here.  */
 
@@ -160,7 +160,7 @@ NCwritereply(CHANNEL *cp, const char *text)
     WCHANappend(cp, text, strlen(text));	/* Text in buffer. */
     WCHANappend(cp, NCterm, strlen(NCterm));	/* Add CR LF to text. */
 
-    if (i == 0) {	/* If only data, then try to write directly. */
+    if (i == 0) {	/* If only new data, then try to write directly. */
 	i = write(cp->fd, &bp->data[bp->used], bp->left);
 	if (Tracing || cp->Tracing)
 	    syslog(L_TRACE, "%s NCwritereply %d=write(%d, \"%.15s\", %lu)",
@@ -173,10 +173,14 @@ NCwritereply(CHANNEL *cp, const char *text)
 	    bp->used = bp->left = 0;
 	    NCwritedone(cp);
 	} else {
-            bp->left -= i;
+            if (i > 0) {
+                bp->left -= i;
+            }
             i = 0;
         }
-    } else i = 0;
+    } else {
+        i = 0;
+    }
     if (i <= 0) {	/* Write failed, queue it for later. */
 	WCHANadd(cp);
     }
@@ -604,7 +608,7 @@ NCcapabilities(CHANNEL *cp)
     }
 
     if (cp->IsAuthenticated && !cp->Nolist) {
-        WCHANappend(cp, "LIST ACTIVE ACTIVE.TIMES NEWSGROUPS", 35);
+        WCHANappend(cp, "LIST ACTIVE ACTIVE.TIMES MOTD NEWSGROUPS", 40);
         WCHANappend(cp, NCterm, strlen(NCterm));
     }
 
@@ -678,10 +682,10 @@ NCihave(CHANNEL *cp)
         if (cp->Sendid.size < msglen) {
             if (cp->Sendid.size > 0)
                 free(cp->Sendid.data);
-            if (msglen > MAXHEADERSIZE)
+            if (msglen > MED_BUFFER)
                 cp->Sendid.size = msglen;
             else
-                cp->Sendid.size = MAXHEADERSIZE;
+                cp->Sendid.size = MED_BUFFER;
             cp->Sendid.data = xmalloc(cp->Sendid.size);
         }
         snprintf(cp->Sendid.data, cp->Sendid.size, "%d %.200s",
@@ -705,10 +709,10 @@ NCihave(CHANNEL *cp)
         if (cp->Sendid.size < msglen) {
 	    if (cp->Sendid.size > 0)
 		free(cp->Sendid.data);
-	    if (msglen > MAXHEADERSIZE)
+	    if (msglen > MED_BUFFER)
 		cp->Sendid.size = msglen;
 	    else
-		cp->Sendid.size = MAXHEADERSIZE;
+		cp->Sendid.size = MED_BUFFER;
 	    cp->Sendid.data = xmalloc(cp->Sendid.size);
 	}
 	snprintf(cp->Sendid.data, cp->Sendid.size, "%d %.200s",
@@ -801,6 +805,7 @@ NClist(CHANNEL *cp)
     char        *p, *path, *save;
     char        savec;
     char        *buff = NULL;
+    bool        checkutf8 = false;
 
     cp->Start = cp->Next;
 
@@ -845,7 +850,7 @@ NClist(CHANNEL *cp)
             free(buff);
 	    return;
 	} else {
-            xasprintf(&buff, "%d Descriptions in form \"group description\"",
+            xasprintf(&buff, "%d Newsgroup descriptions in form \"group description\"",
                       NNTP_OK_LIST);
             NCwritereply(cp, buff);
             free(buff);
@@ -855,13 +860,37 @@ NClist(CHANNEL *cp)
 	qp = QIOopen(path);
         free(path);
 	if (qp == NULL) {
-            xasprintf(&buff, "%d No list of creation times available",
+            xasprintf(&buff, "%d No list of newsgroup creation times available",
                       NNTP_ERR_UNAVAILABLE);
             NCwritereply(cp, buff);
             free(buff);
 	    return;
 	} else {
-            xasprintf(&buff, "%d Group creations in form \"name time who\"",
+            xasprintf(&buff, "%d Newsgroup creation times in form \"group time who\"",
+                      NNTP_OK_LIST);
+            NCwritereply(cp, buff);
+            free(buff);
+        }
+    } else if (strcasecmp(cp->av[1], "MOTD") == 0) {
+        checkutf8 = true;
+        if (cp->ac > 2) {
+            xasprintf(&buff, "%d Unexpected wildmat or argument",
+                      NNTP_ERR_SYNTAX);
+            NCwritereply(cp, buff);
+            free(buff);
+            return;
+        }
+        path = concatpath(innconf->pathetc, INN_PATH_MOTD_INND);
+        qp = QIOopen(path);
+        free(path);
+        if (qp == NULL) {
+            xasprintf(&buff, "%d No message of the day available",
+                      NNTP_ERR_UNAVAILABLE);
+            NCwritereply(cp, buff);
+            free(buff);
+            return;
+        } else {
+            xasprintf(&buff, "%d Message of the day text in UTF-8",
                       NNTP_OK_LIST);
             NCwritereply(cp, buff);
             free(buff);
@@ -882,6 +911,14 @@ NClist(CHANNEL *cp)
             continue;
         }
 
+        if (checkutf8) {
+            if (!is_valid_utf8(p)) {
+                syslog(L_ERROR, "%s NClist bad encoding in %s (UTF-8 expected)",
+                       CHANname(cp), cp->av[1]);
+                continue;
+            }
+        }
+
         /* Check whether the newsgroup matches the wildmat pattern,
          * if given. */
         if (cp->ac == 3) {
@@ -1581,7 +1618,6 @@ NCproc(CHANNEL *cp)
 	cp->State == CSeatarticle) {
 	/* adjust offset only in CSgetheader, CSgetbody or CSeatarticle */
 	data->CurHeader -= cp->Start;
-	data->LastCRLF -= cp->Start;
 	data->Body -= cp->Start;
 	if (data->BytesHeader != NULL)
 	  data->BytesHeader -= cp->Start;
@@ -1766,10 +1802,10 @@ NCcheck(CHANNEL *cp)
     if (cp->Sendid.size < msglen) {
         if (cp->Sendid.size > 0)
             free(cp->Sendid.data);
-        if (msglen > MAXHEADERSIZE)
+        if (msglen > MED_BUFFER)
             cp->Sendid.size = msglen;
         else
-            cp->Sendid.size = MAXHEADERSIZE;
+            cp->Sendid.size = MED_BUFFER;
         cp->Sendid.data = xmalloc(cp->Sendid.size);
     }
     if (!IsValidMessageID(cp->av[1], false)) {
@@ -1795,7 +1831,8 @@ NCcheck(CHANNEL *cp)
         return;
     } else if (Mode == OMpaused) {
         cp->Check_deferred++;
-        xasprintf(&buff, "%d %s", NNTP_FAIL_CHECK_DEFER, ModeReason);
+        xasprintf(&buff, "%d %s %s", NNTP_FAIL_CHECK_DEFER, cp->av[1],
+                  ModeReason);
         NCwritereply(cp, buff);
         free(buff);
         return;
@@ -1916,10 +1953,10 @@ NCtakethis(CHANNEL *cp)
     if (cp->Sendid.size < msglen) {
         if (cp->Sendid.size > 0)
             free(cp->Sendid.data);
-        if (msglen > MAXHEADERSIZE)
+        if (msglen > MED_BUFFER)
             cp->Sendid.size = msglen;
         else
-            cp->Sendid.size = MAXHEADERSIZE;
+            cp->Sendid.size = MED_BUFFER;
         cp->Sendid.data = xmalloc(cp->Sendid.size);
     }
     /* Save ID for later NACK or ACK. */
