@@ -1,4 +1,4 @@
-/*  $Id: article.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: article.c 9462 2013-03-18 18:38:34Z iulius $
 **
 **  The Article class for innfeed.
 **
@@ -238,7 +238,7 @@ Article newArticle (const char *filename, const char *msgid)
               newArt->fname) ;
       
       newArt->refCount++ ;
-      d_printf (2,"Reusing existing article for %s\nx",msgid) ;
+      d_printf (2,"Reusing existing article for %s\n",msgid) ;
     }
   TMRstop(TMR_NEWARTICLE);
   return newArt ;
@@ -815,9 +815,7 @@ static void appendBuffer (Buffer b, Buffer **buffs, int *newSpot, int *curLen)
      characters as needed */
 static bool prepareArticleForNNTP (Article article)
 {
-  static Buffer dotFirstBuffer ;
   static Buffer dotBuffer ;
-  static Buffer crlfBuffer ;
   Buffer *nntpBuffs = NULL ;
   int buffLen = 0 ;
   int buffIdx = 0 ;
@@ -841,8 +839,6 @@ static bool prepareArticleForNNTP (Article article)
   if (dotBuffer == NULL)
     {
       dotBuffer = newBufferByCharP (".\r\n",3,3) ;
-      dotFirstBuffer = newBufferByCharP ("\r\n.",3,3) ;
-      crlfBuffer = newBufferByCharP ("\r\n",2,2) ;
     }
 
   /* overlay a set of buffers on top of the articles contents buffer. This
