@@ -1,4 +1,4 @@
-/* $Id: newsuser.c 9981 2016-01-10 14:59:10Z iulius $
+/* $Id: newsuser.c 9855 2015-05-14 13:13:11Z iulius $
  *
  * Ensure running as "news" user/group.
  *
@@ -69,16 +69,14 @@ ensure_news_user(bool may_setuid)
     if (geteuid() == 0) {
         if (! may_setuid) {
             /* NB:  mustn't be run as root, unless "may_setuid" is true. */
-            die("must be run as %s, not as root",
-                innconf != NULL ? innconf->runasuser : RUNASUSER);
+            die("must be run as %s, not as root", innconf->runasuser);
         }
         if (setuid(uid) < 0) {
             sysdie("failed to setuid");
         }
     }
     if (geteuid() != uid || getuid() != uid) {
-        die("must be run as %s",
-            innconf != NULL ? innconf->runasuser : RUNASUSER);
+        die("must be run as %s", innconf->runasuser);
     }
 }
 
@@ -97,8 +95,7 @@ ensure_news_grp(bool may_setgid)
         }
     }
     if (getegid() != gid || getgid() != gid) {
-        die ("must be run as %s group",
-             innconf != NULL ? innconf->runasgroup : RUNASGROUP);
+        die ("must be run as %s group", innconf->runasgroup);
     }
 }
 
