@@ -1,4 +1,4 @@
-/*  $Id: xwrite.c,v 1.2.6.1 2002/03/25 19:48:44 rra Exp $
+/*  $Id: xwrite.c 7585 2006-11-21 09:37:51Z eagle $
 **
 **  write and writev replacements to handle partial writes.
 **
@@ -29,8 +29,7 @@
 #include <errno.h>
 #include <sys/uio.h>
 
-#include "libinn.h"
-#include "macros.h"
+#include "inn/libinn.h"
 
 /* If we're running the test suite, call testing versions of the write
    functions.  #undef pwrite first because large file support may define a
@@ -52,6 +51,9 @@ xwrite(int fd, const void *buffer, size_t size)
     ssize_t status;
     int count = 0;
 
+    if (size == 0)
+	return 0;
+
     /* Abort the write if we try ten times with no forward progress. */
     for (total = 0; total < size; total += status) {
         if (++count > 10)
@@ -75,6 +77,9 @@ xpwrite(int fd, const void *buffer, size_t size, off_t offset)
     ssize_t status;
     int count = 0;
 
+    if (size == 0)
+	return 0;
+
     /* Abort the write if we try ten times with no forward progress. */
     for (total = 0; total < size; total += status) {
         if (++count > 10)
@@ -100,10 +105,16 @@ xwritev(int fd, const struct iovec iov[], int iovcnt)
     int iovleft, i, count;
     struct iovec *tmpiov;
 
+    if (iovcnt == 0)
+	return 0;
+
     /* Get a count of the total number of bytes in the iov array. */
     for (total = 0, i = 0; i < iovcnt; i++)
         total += iov[i].iov_len;
 
+    if (total == 0)
+	return 0;
+
     /* First, try just writing it all out.  Most of the time this will
        succeed and save us lots of work.  Abort the write if we try ten times 
        with no forward progress. */
@@ -129,7 +140,7 @@ xwritev(int fd, const struct iovec iov[], int iovcnt)
     for (i = 0; offset >= (size_t) iov[i].iov_len; i++)
         offset -= iov[i].iov_len;
     iovleft = iovcnt - i;
-    tmpiov = NEW(struct iovec, iovleft);
+    tmpiov = xmalloc(iovleft * sizeof(struct iovec));
     memcpy(tmpiov, iov + i, iovleft * sizeof(struct iovec));
 
     /* status now contains the offset into the first iovec struct in tmpiov.
