@@ -1,4 +1,4 @@
-/*  $Id: article.c 8519 2009-06-20 12:20:03Z iulius $
+/*  $Id: article.c 9860 2015-05-14 13:32:30Z iulius $
 **
 **  Article-related routines.
 */
@@ -40,9 +40,6 @@ typedef struct _SENDDATA {
     const char *Item;
 } SENDDATA;
 
-static char		ARTnotingroup[] = NNTP_NOTINGROUP;
-static char		ARTnoartingroup[] = NNTP_NOARTINGRP;
-static char		ARTnocurrart[] = NNTP_NOCURRART;
 static ARTHANDLE        *ARThandle = NULL;
 static SENDDATA		SENDbody = {
     STbody,	NNTP_OK_BODY,		"body"
@@ -65,6 +62,8 @@ PushIOvHelper(struct iovec* vec, int* countp)
 {
     int result = 0;
 
+    TMRstart(TMR_NNTPWRITE);
+
 #ifdef HAVE_SASL
     if (sasl_conn && sasl_ssf) {
 	int i;
@@ -75,8 +74,6 @@ PushIOvHelper(struct iovec* vec, int* countp)
     } else {
 #endif /* HAVE_SASL */
 
-        TMRstart(TMR_NNTPWRITE);
-
 #ifdef HAVE_SSL
 	if (tls_conn) {
 Again:
@@ -100,12 +97,12 @@ Again:
 #endif /* HAVE_SSL */
 	    result = xwritev(STDOUT_FILENO, vec, *countp);
 
-        TMRstop(TMR_NNTPWRITE);
-
 #ifdef HAVE_SASL
     }
 #endif
 
+    TMRstop(TMR_NNTPWRITE);
+
     if (result == -1) {
 	/* We can't recover, since we can't resynchronise with our
 	 * peer. */
@@ -244,7 +241,8 @@ ARTinstorebytoken(TOKEN token)
     if (PERMaccessconf->nnrpdoverstats) {
 	gettimeofday(&stv, NULL);
     }
-    art = SMretrieve(token, RETR_STAT); /* XXX This isn't really overstats, is it? */
+    art = SMretrieve(token, RETR_STAT);
+    /* XXX This isn't really overstats, is it? */
     if (PERMaccessconf->nnrpdoverstats) {
 	gettimeofday(&etv, NULL);
 	OVERartcheck+=(etv.tv_sec - stv.tv_sec) * 1000;
@@ -338,12 +336,10 @@ ARTsendmmap(SENDTYPE what)
 {
     const char		*p, *q, *r;
     const char		*s, *path, *xref, *endofpath;
-    long		bytecount;
     char		lastchar;
 
     ARTcount++;
     GRParticles++;
-    bytecount = 0;
     lastchar = -1;
 
     /* Get the headers and detect if wire format. */
@@ -373,7 +369,7 @@ ARTsendmmap(SENDTYPE what)
 
     /* q points to the start of the article buffer, p to the end of it. */
     if (VirtualPathlen > 0 && (what != STbody)) {
-        path = wire_findheader(ARThandle->data, ARThandle->len, "Path");
+        path = wire_findheader(ARThandle->data, ARThandle->len, "Path", true);
         if (path == NULL) {
 	    SendIOv(".\r\n", 3);
 	    ARTgetsize += 3;
@@ -381,7 +377,7 @@ ARTsendmmap(SENDTYPE what)
 	    ARTget++;
 	    return;
 	} else {
-            xref = wire_findheader(ARThandle->data, ARThandle->len, "Xref");
+            xref = wire_findheader(ARThandle->data, ARThandle->len, "Xref", true);
             if (xref == NULL) {
                 SendIOv(".\r\n", 3);
                 ARTgetsize += 3;
@@ -406,11 +402,12 @@ ARTsendmmap(SENDTYPE what)
 	    return;
 	}
 	/* r points to the first space in the Xref: header. */
+
 	for (s = path, lastchar = '\0';
 	    s + VirtualPathlen + 1 < endofpath;
 	    lastchar = *s++) {
 	    if ((lastchar != '\0' && lastchar != '!') || *s != *VirtualPath ||
-		strncmp(s, VirtualPath, VirtualPathlen - 1) != 0)
+		strncasecmp(s, VirtualPath, VirtualPathlen - 1) != 0)
 		continue;
 	    if (*(s + VirtualPathlen - 1) != '\0' &&
 		*(s + VirtualPathlen - 1) != '!')
@@ -430,9 +427,11 @@ ARTsendmmap(SENDTYPE what)
 	        SendIOv(s, p - s);
 	    }
 	} else {
+            /* Double the '!' (thus, adding one '!') in Path: header. */
 	    if (xref > path) {
 	        SendIOv(q, path - q);
 	        SendIOv(VirtualPath, VirtualPathlen);
+                SendIOv("!", 1);
 	        SendIOv(path, xref - path);
 	        SendIOv(VirtualPath, VirtualPathlen - 1);
 	        SendIOv(r, p - r);
@@ -441,6 +440,7 @@ ARTsendmmap(SENDTYPE what)
 	        SendIOv(VirtualPath, VirtualPathlen - 1);
 	        SendIOv(r, path - r);
 	        SendIOv(VirtualPath, VirtualPathlen);
+                SendIOv("!", 1);
 	        SendIOv(path, p - path);
 	    }
 	}
@@ -468,10 +468,10 @@ ARTsendmmap(SENDTYPE what)
 **  Return the header from the specified file, or NULL if not found.
 */
 char *
-GetHeader(const char *header)
+GetHeader(const char *header, bool stripspaces)
 {
     const char		*p, *q, *r, *s, *t;
-    char		*w, prevchar;
+    char		*w, *wnew, prevchar;
     /* Bogus value here to make sure that it isn't initialized to \n. */
     char		lastchar = ' ';
     const char		*limit;
@@ -492,9 +492,12 @@ GetHeader(const char *header)
 	}
 	if ((lastchar == '\n') || (p == ARThandle->data)) {
 	    headerlen = strlen(header);
-	    if (strncasecmp(p, header, headerlen) == 0 && p[headerlen] == ':') {
-		for (; (p < limit) && !isspace((int)*p) ; p++);
-		for (; (p < limit) && isspace((int)*p) ; p++);
+	    if (strncasecmp(p, header, headerlen) == 0 && p[headerlen] == ':' &&
+                ISWHITE(p[headerlen+1])) {
+                p += headerlen + 2;
+                if (stripspaces) {
+                    for (; (p < limit) && ISWHITE(*p) ; p++);
+                }
 		for (q = p; q < limit; q++) 
 		    if ((*q == '\r') || (*q == '\n')) {
 			/* Check for continuation header lines. */
@@ -506,7 +509,7 @@ GetHeader(const char *header)
 				    break;
 			    }
 			    if ((*t == '\t' || *t == ' ')) {
-				for (; (t < limit) && isspace((int)*t) ; t++);
+				for (; (t < limit) && ISWHITE(*t) ; t++);
 				q = t;
 			    } else {
 				break;
@@ -522,11 +525,13 @@ GetHeader(const char *header)
 		else if (strncasecmp("Xref", header, headerlen) == 0)
 		    xrefheader = true;
 		if (retval == NULL) {
-		    retlen = q - p + VirtualPathlen + 1;
+                    /* Possibly add '!' (a second one) at the end of the virtual path.
+                     * So it is +2 because of '\0'. */
+		    retlen = q - p + VirtualPathlen + 2;
 		    retval = xmalloc(retlen);
 		} else {
-		    if ((q - p + VirtualPathlen + 1) > retlen) {
-			retlen = q - p + VirtualPathlen + 1;
+		    if ((q - p + VirtualPathlen + 2) > retlen) {
+			retlen = q - p + VirtualPathlen + 2;
                         retval = xrealloc(retval, retlen);
 		    }
 		}
@@ -543,7 +548,7 @@ GetHeader(const char *header)
 			prevchar = *s++) {
 			if ((prevchar != '\0' && prevchar != '!') ||
 			    *s != *VirtualPath ||
-			    strncmp(s, VirtualPath, VirtualPathlen - 1) != 0)
+			    strncasecmp(s, VirtualPath, VirtualPathlen - 1) != 0)
 			    continue;
 			if (*(s + VirtualPathlen - 1) != '\0' &&
 			    *(s + VirtualPathlen - 1) != '!')
@@ -555,15 +560,17 @@ GetHeader(const char *header)
 			*(retval + (int)(q - s)) = '\0';
 		    } else {
 			memcpy(retval, VirtualPath, VirtualPathlen);
-			memcpy(retval + VirtualPathlen, p, q - p);
-			*(retval + (int)(q - p) + VirtualPathlen) = '\0';
+                        *(retval + VirtualPathlen) = '!';
+			memcpy(retval + VirtualPathlen + 1, p, q - p);
+			*(retval + (int)(q - p) + VirtualPathlen + 1) = '\0';
 		    }
 		} else if (xrefheader && (VirtualPathlen > 0)) {
 		    if ((r = memchr(p, ' ', q - p)) == NULL)
 			return NULL;
-		    for (; (r < q) && isspace((int)*r) ; r++);
+		    for (; (r < q) && isspace((unsigned char) *r) ; r++);
 		    if (r == q)
 			return NULL;
+                    /* Copy the virtual path without its final '!'. */
 		    memcpy(retval, VirtualPath, VirtualPathlen - 1);
 		    memcpy(retval + VirtualPathlen - 1, r - 1, q - r + 1);
 		    *(retval + (int)(q - r) + VirtualPathlen) = '\0';
@@ -571,9 +578,19 @@ GetHeader(const char *header)
 		    memcpy(retval, p, q - p);
 		    *(retval + (int)(q - p)) = '\0';
 		}
-		for (w = retval; *w; w++)
-		    if (*w == '\n' || *w == '\r')
-			*w = ' ';
+		for (w = retval, wnew = retval; *w; w++) {
+                    if (*w == '\r' && w[1] == '\n') {
+                        w++;
+                        continue;
+                    }
+		    if (*w == '\0' || *w == '\t' || *w == '\r' || *w == '\n') {
+			*wnew = ' ';
+                    } else {
+                        *wnew = *w;
+                    }
+                    wnew++;
+                }
+                *wnew = '\0';
 		return retval;
 	    }
 	}
@@ -596,14 +613,14 @@ CMDfetch(int ac, char *av[])
     ARTNUM		tart;
     bool                final = false;
 
-    mid = (ac > 1 && IsValidMessageID(av[1]));
+    mid = (ac > 1 && IsValidMessageID(av[1], true));
 
     /* Check the syntax of the arguments first. */
     if (ac > 1 && !IsValidArticleNumber(av[1])) {
         /* It is better to check for a number before a message-ID because
          * '<' could have been forgotten and the error message should then
          * report a syntax error in the message-ID. */
-        if (CTYPE(isdigit, av[1][0])) {
+        if (isdigit((unsigned char) av[1][0])) {
             Reply("%d Syntax error in article number\r\n", NNTP_ERR_SYNTAX);
             return;
         } else if (!mid) {
@@ -633,6 +650,12 @@ CMDfetch(int ac, char *av[])
 	break;
     }
 
+    /* Trying to read. */
+    if (GRPcount == 0 && !mid) {
+        Reply("%d Not in a newsgroup\r\n", NNTP_FAIL_NO_GROUP);
+        return;
+    }
+
     /* Check authorizations.  If an article number is requested
      * (not a message-ID), we check whether the group is still readable. */
     if (!ok || (!mid && PERMgroupmadeinvalid)) {
@@ -644,7 +667,7 @@ CMDfetch(int ac, char *av[])
     /* Requesting by message-ID? */
     if (mid) {
 	if (!ARTopenbyid(av[1], &art, final)) {
-	    Reply("%d No such article\r\n", NNTP_FAIL_NOTFOUND);
+	    Reply("%d No such article\r\n", NNTP_FAIL_MSGID_NOTFOUND);
 	    return;
 	}
 	if (!PERMartok()) {
@@ -662,43 +685,38 @@ CMDfetch(int ac, char *av[])
 	return;
     }
 
-    /* Trying to read. */
-    if (GRPcount == 0) {
-	Reply("%s\r\n", ARTnotingroup);
-	return;
-    }
-
     /* Default is to get current article, or specified article. */
     if (ac == 1) {
-	if (ARTnumber < ARTlow || ARTnumber > ARThigh) {
-	    Reply("%s\r\n", ARTnocurrart);
-	    return;
-	}
-	snprintf(buff, sizeof(buff), "%lu", ARTnumber);
-	tart=ARTnumber;
-    }
-    else {
+        if (ARTnumber < ARTlow || ARTnumber > ARThigh) {
+            Reply("%d Current article number %lu is invalid\r\n",
+                  NNTP_FAIL_ARTNUM_INVALID, ARTnumber);
+            return;
+        }
+        snprintf(buff, sizeof(buff), "%lu", ARTnumber);
+        tart = ARTnumber;
+    } else {
         /* We have already checked that the article number is valid. */
-	strlcpy(buff, av[1], sizeof(buff));
-	tart=(ARTNUM)atol(buff);
+        strlcpy(buff, av[1], sizeof(buff));
+        tart = (ARTNUM)atol(buff);
     }
 
     /* Open the article and send the reply. */
-    if (!ARTopen(atol(buff))) {
-        Reply("%s\r\n", ARTnoartingroup);
+    if (!ARTopen(tart)) {
+        Reply("%d No such article number %lu\r\n", NNTP_FAIL_ARTNUM_NOTFOUND, tart);
+        return;
+    }
+    if ((msgid = GetHeader("Message-ID", true)) == NULL) {
+        ARTclose();
+        Reply("%d No such article number %lu\r\n", NNTP_FAIL_ARTNUM_NOTFOUND, tart);
         return;
     }
     if (ac > 1)
-	ARTnumber = tart;
-    if ((msgid = GetHeader("Message-ID")) == NULL) {
-        ARTclose();
-        Reply("%s\r\n", ARTnoartingroup);
-	return;
-    }
+        ARTnumber = tart;
+
     /* A message-ID does not have more than 250 octets. */
     Reply("%d %s %.250s %s\r\n", what->ReplyCode, buff, msgid, what->Item); 
     if (what->Type != STstat)
-	ARTsendmmap(what->Type);
+        ARTsendmmap(what->Type);
     ARTclose();
 }
 
@@ -714,6 +732,12 @@ CMDnextlast(int ac UNUSED, char *av[])
     bool next;
     const char *message;
 
+    /* Trying to read. */
+    if (GRPcount == 0) {
+        Reply("%d Not in a newsgroup\r\n", NNTP_FAIL_NO_GROUP);
+        return;
+    }
+
     /* No syntax to check.  Only check authorizations. */
     if (!PERMcanread || PERMgroupmadeinvalid) {
 	Reply("%d Read access denied\r\n",
@@ -721,14 +745,10 @@ CMDnextlast(int ac UNUSED, char *av[])
 	return;
     }
 
-    /* Trying to read. */
-    if (GRPcount == 0) {
-	Reply("%s\r\n", ARTnotingroup);
-	return;
-    }
     if (ARTnumber < ARTlow || ARTnumber > ARThigh) {
-	Reply("%s\r\n", ARTnocurrart);
-	return;
+        Reply("%d Current article number %lu is invalid\r\n",
+              NNTP_FAIL_ARTNUM_INVALID, ARTnumber);
+        return;
     }
 
     /* NEXT? */
@@ -755,11 +775,11 @@ CMDnextlast(int ac UNUSED, char *av[])
         }
         if (!ARTopen(ARTnumber))
             continue;
-        msgid = GetHeader("Message-ID");
+        msgid = GetHeader("Message-ID", true);
         ARTclose();
     } while (msgid == NULL);
 
-    Reply("%d %d %s Article retrieved; request text separately\r\n",
+    Reply("%d %lu %s Article retrieved; request text separately\r\n",
 	   NNTP_OK_STAT, ARTnumber, msgid);
 }
 
@@ -794,13 +814,14 @@ CMDgetrange(int ac, char *av[], ARTRANGE *rp, bool *DidReply)
     *DidReply = false;
 
     if (ac == 1) {
-	/* No arguments, do only current article. */
-	if (ARTnumber < ARTlow || ARTnumber > ARThigh) {
-	    Reply("%s\r\n", ARTnocurrart);
-	    *DidReply = true;
-	    return false;
-	}
-	rp->High = rp->Low = ARTnumber;
+        /* No arguments, do only current article. */
+        if (ARTnumber < ARTlow || ARTnumber > ARThigh) {
+            Reply("%d Current article number %lu is invalid\r\n",
+                  NNTP_FAIL_ARTNUM_INVALID, ARTnumber);
+            *DidReply = true;
+            return false;
+        }
+        rp->High = rp->Low = ARTnumber;
         return true;
     }
 
@@ -852,14 +873,14 @@ vhost_xref(char *p)
 
     space = strchr(p, ' ');
     if (space == NULL) {
-	warn("malformed Xref: `%s'", field);
-	goto fail;
+        warn("malformed Xref: `%s'", p);
+        goto fail;
     }
-    offset = space + 1 - p;
+    offset = space - p;
     space = strchr(p + offset, ' ');
     if (space == NULL) {
-	warn("malformed Xref: `%s'", field);
-	goto fail;
+        warn("malformed Xref: `%s'", p);
+        goto fail;
     }
     field = concat(PERMaccessconf->domain, space, NULL);
  fail:
@@ -867,6 +888,7 @@ vhost_xref(char *p)
     return field;
 }
 
+
 /*
 **  Dump parts of the overview database with the OVER command.
 **  The legacy XOVER is also kept, with its specific behaviour.
@@ -887,7 +909,7 @@ CMDover(int ac, char *av[])
     bool                xover, mid;
 
     xover = (strcasecmp(av[0], "XOVER") == 0);
-    mid = (ac > 1 && IsValidMessageID(av[1]));
+    mid = (ac > 1 && IsValidMessageID(av[1], true));
 
     if (mid && !xover) {
         /* FIXME:  We still do not support OVER MSGID, sorry! */
@@ -902,7 +924,7 @@ CMDover(int ac, char *av[])
         /* It is better to check for a range before a message-ID because
          * '<' could have been forgotten and the error message should then
          * report a syntax error in the message-ID. */
-        if (xover || CTYPE(isdigit, av[1][0]) || av[1][0] == '-') {
+        if (xover || isdigit((unsigned char) av[1][0]) || av[1][0] == '-') {
             Reply("%d Syntax error in range\r\n", NNTP_ERR_SYNTAX);
             return;
         } else if (!mid) {
@@ -911,6 +933,12 @@ CMDover(int ac, char *av[])
         }   
     }
 
+    /* Trying to read. */
+    if (GRPcount == 0) {
+        Reply("%d Not in a newsgroup\r\n", NNTP_FAIL_NO_GROUP);
+        return;
+    }
+
     /* Check authorizations.  If a range is requested (not a message-ID),
      * we check whether the group is still readable. */
     if (!PERMcanread || (!mid && PERMgroupmadeinvalid)) {
@@ -919,12 +947,6 @@ CMDover(int ac, char *av[])
 	return;
     }
 
-    /* Trying to read. */
-    if (GRPcount == 0) {
-	Reply("%s\r\n", ARTnotingroup);
-	return;
-    }
-
     /* Parse range.  CMDgetrange() correctly sets the range when
      * there is no arguments. */
     if (!CMDgetrange(ac, av, &range, &DidReply))
@@ -940,10 +962,10 @@ CMDover(int ac, char *av[])
          * Note that XOVER answers OK. */
         if (ac > 1)
             Reply("%d No articles in %s\r\n",
-                  xover ? NNTP_OK_OVER : NNTP_FAIL_BAD_ARTICLE, av[1]);
+                  xover ? NNTP_OK_OVER : NNTP_FAIL_ARTNUM_NOTFOUND, av[1]);
         else
-            Reply("%d Current article number %d is invalid\r\n",
-                  xover ? NNTP_OK_OVER : NNTP_FAIL_NO_ARTICLE, ARTnumber);
+            Reply("%d No such article number %lu\r\n",
+                  xover ? NNTP_OK_OVER : NNTP_FAIL_ARTNUM_NOTFOUND, ARTnumber);
         if (xover)
             Printf(".\r\n");
         return;
@@ -986,7 +1008,7 @@ CMDover(int ac, char *av[])
                 Reply("%d Overview information for %s follows\r\n",
                       NNTP_OK_OVER, av[1]);
             else
-                Reply("%d Overview information for %d follows\r\n",
+                Reply("%d Overview information for %lu follows\r\n",
                       NNTP_OK_OVER, ARTnumber);
             fflush(stdout);
             HasNotReplied = false;
@@ -1019,6 +1041,7 @@ CMDover(int ac, char *av[])
                 }
                 continue;
             }
+            /* Copy the virtual path without its final '!'. */
 	    if(useIOb) {
 		SendIOb(data, p - data);
 		SendIOb(VirtualPath, VirtualPathlen - 1);
@@ -1052,10 +1075,10 @@ CMDover(int ac, char *av[])
          * Note that XOVER answers OK. */
         if (ac > 1)
             Reply("%d No articles in %s\r\n",
-                  xover ? NNTP_OK_OVER : NNTP_FAIL_BAD_ARTICLE, av[1]);
+                  xover ? NNTP_OK_OVER : NNTP_FAIL_ARTNUM_NOTFOUND, av[1]);
         else
-            Reply("%d Current article number %d is invalid\r\n",
-                  xover ? NNTP_OK_OVER : NNTP_FAIL_NO_ARTICLE, ARTnumber);
+            Reply("%d No such article number %lu\r\n",
+                  xover ? NNTP_OK_OVER : NNTP_FAIL_ARTNUM_NOTFOUND, ARTnumber);
         if (xover)
             Printf(".\r\n");
     } else {
@@ -1106,14 +1129,14 @@ CMDpat(int ac, char *av[])
     bool                hdr, mid;
 
     hdr = (strcasecmp(av[0], "HDR") == 0);
-    mid = (ac > 2 && IsValidMessageID(av[2]));
+    mid = (ac > 2 && IsValidMessageID(av[2], true));
 
     /* Check the syntax of the arguments first. */
     if (ac > 2 && !IsValidRange(av[2])) {
         /* It is better to check for a range before a message-ID because
          * '<' could have been forgotten and the error message should then
          * report a syntax error in the message-ID. */
-        if (CTYPE(isdigit, av[2][0]) || av[2][0] == '-') {
+        if (isdigit((unsigned char) av[2][0]) || av[2][0] == '-') {
             Reply("%d Syntax error in range\r\n", NNTP_ERR_SYNTAX);
             return;
         } else if (!mid) {
@@ -1143,7 +1166,7 @@ CMDpat(int ac, char *av[])
         p++;
         if (strncasecmp(header, ":", 1) == 0 && IsValidHeaderName(p)) {
             Reply("%d Unsupported metadata request\r\n",
-                  NNTP_ERR_UNAVAILABLE, header);
+                  NNTP_ERR_UNAVAILABLE);
             return;
         } else if (!IsValidHeaderName(header)) {
             Reply("%d Syntax error in header name\r\n", NNTP_ERR_SYNTAX);
@@ -1151,6 +1174,12 @@ CMDpat(int ac, char *av[])
         }
     }
 
+    /* Trying to read. */
+    if (GRPcount == 0 && !mid) {
+        Reply("%d Not in a newsgroup\r\n", NNTP_FAIL_NO_GROUP);
+        return;
+    }
+
     /* Check authorizations.  If a range is requested (not a message-ID),
      * we check whether the group is still readable. */
     if (!PERMcanread || (!mid && PERMgroupmadeinvalid)) {
@@ -1168,12 +1197,6 @@ CMDpat(int ac, char *av[])
     do {
 	/* Message-ID specified? */
 	if (mid) {
-	    p = av[2];
-	    if (!ARTopenbyid(p, &artnum, false)) {
-		Reply("%d No such article\r\n", NNTP_FAIL_NOTFOUND);
-		break;
-	    }
-
             /* FIXME:  We do not handle metadata requests by message-ID. */
             if (hdr && (IsMetaBytes || IsMetaLines)) {
                 Reply("%d Metadata requests by message-ID unsupported\r\n",
@@ -1181,6 +1204,12 @@ CMDpat(int ac, char *av[])
                 break;
             }
 
+	    p = av[2];
+	    if (!ARTopenbyid(p, &artnum, false)) {
+		Reply("%d No such article\r\n", NNTP_FAIL_MSGID_NOTFOUND);
+		break;
+	    }
+
             if (!PERMartok()) {
                 ARTclose();
                 Reply("%d Read access denied for this article\r\n",
@@ -1191,7 +1220,7 @@ CMDpat(int ac, char *av[])
 	    Reply("%d Header information for %s follows (from the article)\r\n",
                    hdr ? NNTP_OK_HDR : NNTP_OK_HEAD, av[1]);
 
-	    if ((text = GetHeader(av[1])) != NULL
+	    if ((text = GetHeader(av[1], false)) != NULL
 		&& (!pattern || uwildmat_simple(text, pattern)))
 		Printf("%s %s\r\n", hdr ? "0" : p, text);
             else if (hdr) {
@@ -1204,12 +1233,6 @@ CMDpat(int ac, char *av[])
 	    break;
 	}
 
-        /* Trying to read. */
-	if (GRPcount == 0) {
-	    Reply("%s\r\n", ARTnotingroup);
-	    break;
-	}
-
         /* Parse range.  CMDgetrange() correctly sets the range when
          * there is no arguments. */
         if (!CMDgetrange(ac - 1, av + 1, &range, &DidReply))
@@ -1231,7 +1254,7 @@ CMDpat(int ac, char *av[])
                           hdr ? NNTP_OK_HDR : NNTP_OK_HEAD, av[1]);
                     HasNotReplied = false;
                 }
-		p = GetHeader(header);
+		p = GetHeader(header, false);
 		if (p && (!pattern || uwildmat_simple(p, pattern))) {
 		    snprintf(buff, sizeof(buff), "%lu ", i);
 		    SendIOb(buff, strlen(buff));
@@ -1248,10 +1271,10 @@ CMDpat(int ac, char *av[])
                 if (hdr) {
                     if (ac > 2)
                         Reply("%d No articles in %s\r\n",
-                              NNTP_FAIL_BAD_ARTICLE, av[2]);
+                              NNTP_FAIL_ARTNUM_NOTFOUND, av[2]);
                     else
-                        Reply("%d Current article number %d is invalid\r\n",
-                              NNTP_FAIL_NO_ARTICLE, ARTnumber);
+                        Reply("%d No such article number %lu\r\n",
+                              NNTP_FAIL_ARTNUM_NOTFOUND, ARTnumber);
                 } else {
                     Reply("%d No header information for %s follows (from articles)\r\n",
                           NNTP_OK_HEAD, av[1]);
@@ -1271,10 +1294,10 @@ CMDpat(int ac, char *av[])
             if (hdr) {
                 if (ac > 2)
                     Reply("%d No articles in %s\r\n",
-                          NNTP_FAIL_BAD_ARTICLE, av[2]);
+                          NNTP_FAIL_ARTNUM_NOTFOUND, av[2]);
                 else
-                    Reply("%d Current article number %d is invalid\r\n",
-                          NNTP_FAIL_NO_ARTICLE, ARTnumber);
+                    Reply("%d No such article number %lu\r\n",
+                          NNTP_FAIL_ARTNUM_NOTFOUND, ARTnumber);
             } else {
                 Reply("%d No header information for %s follows (from overview)\r\n",
                       NNTP_OK_HEAD, av[1]);
@@ -1328,10 +1351,10 @@ CMDpat(int ac, char *av[])
             if (hdr) {
                 if (ac > 2)
                     Reply("%d No articles in %s\r\n",
-                          NNTP_FAIL_BAD_ARTICLE, av[2]);
+                          NNTP_FAIL_ARTNUM_NOTFOUND, av[2]);
                 else
-                    Reply("%d Current article number %d is invalid\r\n",
-                          NNTP_FAIL_NO_ARTICLE, ARTnumber);
+                    Reply("%d Current article number %lu is invalid\r\n",
+                          NNTP_FAIL_ARTNUM_INVALID, ARTnumber);
             } else {
                 Reply("%d No header or metadata information for %s follows (from overview)\r\n",
                       NNTP_OK_HEAD, av[1]);
