@@ -1,4 +1,4 @@
-/*  $Id: commands.c 8267 2008-12-27 16:04:03Z iulius $
+/*  $Id: commands.c 9864 2015-05-14 13:41:41Z iulius $
 **
 **  Miscellaneous commands.
 */
@@ -87,8 +87,8 @@ makeGroupInvalid(void) {
 
 /*  Returns:
 **    -1 for problem (such as no such authenticator, etc.).
-**     0 for authentication succeeded.
-**     1 for authentication failed.
+**     1 for authentication succeeded.
+**     0 for authentication failed.
 */
 static char *PERMauthstring;
 
@@ -96,6 +96,7 @@ static int
 PERMgeneric(char *av[], char *accesslist, size_t size)
 {
     char path[BIG_BUFFER], *fields[6], *p;
+    size_t j;
     int i, pan[2], status;
     pid_t pid;
     struct stat stb;
@@ -158,9 +159,9 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
     }
 
     for (i = 0; (pid = fork()) < 0; i++) {
-	if (i == innconf->maxforks) {
-	    Reply("%d Can't fork %s\r\n", NNTP_FAIL_ACTION,
-		strerror(errno));
+        if (i == (long) innconf->maxforks) {
+            Reply("%d Can't fork %s\r\n", NNTP_FAIL_ACTION,
+                  strerror(errno));
 	    syslog(L_FATAL, "can't fork %s %m", av[0]);
 	    return -1;
 	}
@@ -176,8 +177,8 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
 	/* stderr goes down the pipe. */
 	if (pan[PIPE_WRITE] != STDERR_FILENO) {
 	    if ((i = dup2(pan[PIPE_WRITE], STDERR_FILENO)) != STDERR_FILENO) {
-		syslog(L_FATAL, "can't dup2 %d to %d got %d %m",
-		    pan[PIPE_WRITE], STDERR_FILENO, i);
+                syslog(L_FATAL, "can't dup2 %d to %d got %d %m",
+                       pan[PIPE_WRITE], STDERR_FILENO, i);
 		_exit(1);
 	    }
 	    close(pan[PIPE_WRITE]);
@@ -201,7 +202,7 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
 
     waitpid(pid, &status, 0);
     if (!WIFEXITED(status) || WEXITSTATUS(status) != 0)
-        return 1;
+        return 0;
 
     if ((p = strchr(path, '\n')) != NULL)
 	*p = '\0';
@@ -213,12 +214,25 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
 
     //syslog(L_NOTICE, "%s (%ld) returned: %d %s %d\n", av[0], (long) pid, i, path, status);
     /* Split "host:permissions:user:pass:groups" into fields. */
-    for (fields[0] = path, i = 0, p = path; *p; p++)
-	if (*p == ':') {
-	    *p = '\0';
-	    fields[++i] = p + 1;
+    for (fields[0] = path, j = 0, p = path; *p; p++)
+        if (*p == ':') {
+            *p = '\0';
+            ++j;
+            if (j < ARRAY_SIZE(fields)) {
+                fields[j] = p + 1;
+            } else {
+                Reply("%d Program error occurred\r\n", NNTP_FAIL_ACTION);
+                syslog(L_FATAL, "over-long response from %s", av[0]);
+                return -1;
+            }
 	}
 
+    if (j < 4) {
+        Reply("%d Program error occurred\r\n", NNTP_FAIL_ACTION);
+        syslog(L_FATAL, "short response from %s", av[0]);
+        return -1;
+    }
+
     PERMcanread = strchr(fields[1], 'R') != NULL;
     PERMcanpost = strchr(fields[1], 'P') != NULL;
     PERMaccessconf->allowapproved = strchr(fields[1], 'A') != NULL;
@@ -232,7 +246,7 @@ PERMgeneric(char *av[], char *accesslist, size_t size)
     //for (i = 0; fields[i] && i < 5; i++)
     //    syslog(L_NOTICE, "fields[%d] = %s\n", i, fields[i]);
 
-    return 0;
+    return 1;
 }
 
 
@@ -252,6 +266,10 @@ CMDauthinfo(int ac, char *av[])
     if (strcasecmp(av[1], "GENERIC") == 0) {
 	char *logrec = Glom(av);
 
+        /* Go on parsing the command line. */
+        ac--;
+        ac += reArgify(av[ac], &av[ac], -1, true);
+
 	strlcpy(PERMuser, "<none>", sizeof(PERMuser));
 
         /* Arguments are checked by PERMgeneric(). */
@@ -282,10 +300,14 @@ CMDauthinfo(int ac, char *av[])
 	}
     } else if (strcasecmp(av[1], "SASL") == 0) {
 #ifdef HAVE_SASL
+        /* Go on parsing the command line. */
+        ac--;
+        ac += reArgify(av[ac], &av[ac], -1, true);
+
         /* Arguments are checked by SASLauth(). */
 	SASLauth(ac, av);
 #else
-        Reply("%d SASL authentication unsupported\r\n", NNTP_ERR_UNAVAILABLE);
+        Reply("%d SASL authentication unsupported\r\n", NNTP_ERR_SYNTAX);
         return;
 #endif /* HAVE_SASL */
     } else {
@@ -309,10 +331,6 @@ CMDauthinfo(int ac, char *av[])
             }
 #endif
 
-            if (ac > 3) {
-                Reply("%d No whitespace allowed in username\r\n", NNTP_ERR_SYNTAX);
-                return;
-            }
             strlcpy(User, av[2], sizeof(User));
             Reply("%d Enter password\r\n", NNTP_CONT_AUTHINFO);
             return;
@@ -349,11 +367,6 @@ CMDauthinfo(int ac, char *av[])
             return;
         }
 
-        if (ac > 3) {
-            Reply("%d No whitespace allowed in password\r\n", NNTP_ERR_SYNTAX);
-            return;
-        }
-
         /* There is a cached username and a password is provided. */
         strlcpy(Password, av[2], sizeof(Password));
 
@@ -414,7 +427,7 @@ CMDdate(int ac UNUSED, char *av[] UNUSED)
         Reply("%d Can't get time, %s\r\n", NNTP_FAIL_ACTION, strerror(errno));
         return;
     }
-    Reply("%d %04.4d%02.2d%02.2d%02.2d%02.2d%02.2d\r\n",
+    Reply("%d %04d%02d%02d%02d%02d%02d\r\n",
           NNTP_INFO_DATE,
           gmt->tm_year + 1900, gmt->tm_mon + 1, gmt->tm_mday,
           gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
@@ -538,6 +551,12 @@ CMDnewgroups(int ac, char *av[])
     if ((qp = QIOopen(ACTIVE)) == NULL) {
 	syslog(L_ERROR, "%s can't fopen %s %m", Client.host, ACTIVE);
 	Reply("%d Can't open active file\r\n", NNTP_FAIL_ACTION);
+
+        for (i = 0; i < numgroups; i++) {
+            free(grouplist[i].name);
+        }
+        free(grouplist);
+
 	return;
     }
     qsort(grouplist, numgroups, sizeof(GROUPDATA), GroupCompare);
@@ -597,7 +616,7 @@ CMDpost(int ac, char *av[])
     ihave = (strcasecmp(av[0], "IHAVE") == 0);
 
     /* Check whether the message-ID is valid for IHAVE. */
-    if (ihave && ac == 2 && !IsValidMessageID(av[1])) {
+    if (ihave && ac == 2 && !IsValidMessageID(av[1], true)) {
         Reply("%d Syntax error in message-ID\r\n", NNTP_ERR_SYNTAX);
         return;
     }
@@ -611,7 +630,9 @@ CMDpost(int ac, char *av[])
     }
     if (!ihave && !PERMcanpost) {
 	syslog(L_NOTICE, "%s noperm post without permission", Client.host);
-	Reply("%d Posting not allowed\r\n", NNTP_FAIL_POST_AUTH);
+        Reply("%d Posting not allowed\r\n",
+              PERMcanauthenticate && PERMcanpostgreeting ?
+              NNTP_FAIL_AUTH_NEEDED : NNTP_FAIL_POST_AUTH);
 	return;
     }
 
@@ -621,42 +642,40 @@ CMDpost(int ac, char *av[])
 	backoff_inited = true;
     }
 
-    /* Dave's posting limiter.  Limit postings to a certain rate
+    /* Dave's posting limiter.  Limit postings to a certain rate.
      * And now we support multiprocess rate limits.  Questions?
-     * E-mail <dave@jetcafe.org>.
-     */
+     * E-mail <dave@jetcafe.org>. */
     if (BACKOFFenabled) {
-
-      /* Acquire lock (this could be in RateLimit but that would
-       * invoke the spaghetti factor). 
-       */
-      if ((path = (char *) PostRecFilename(Client.ip,PERMuser)) == NULL) {
-        Reply("%s\r\n", ihave ? NNTP_RESENDIT_LATER : NNTP_CANTPOST);
-        return;
-      }
+        /* Acquire lock (this could be in RateLimit but that would
+         * invoke the spaghetti factor). */
+        if ((path = (char *) PostRecFilename(Client.ip, PERMuser)) == NULL) {
+            Reply("%d Retry later\r\n",
+                  ihave ? NNTP_FAIL_IHAVE_DEFER : NNTP_FAIL_POST_AUTH);
+            return;
+        }
       
-      if (LockPostRec(path) == 0) {
-        syslog(L_ERROR, "%s error write locking '%s'",
-               Client.host, path);
-        Reply("%s\r\n", ihave ? NNTP_RESENDIT_LATER : NNTP_CANTPOST);
-        return;
-      }
+        if (LockPostRec(path) == 0) {
+            syslog(L_ERROR, "%s error write locking '%s'",
+                   Client.host, path);
+            Reply("%d Retry later\r\n",
+                  ihave ? NNTP_FAIL_IHAVE_DEFER : NNTP_FAIL_POST_AUTH);
+            return;
+        }
       
-      if (!RateLimit(&sleeptime,path)) {
-	syslog(L_ERROR, "%s can't check rate limit info", Client.host);
-	Reply("%s\r\n", ihave ? NNTP_RESENDIT_LATER : NNTP_CANTPOST);
+        if (!RateLimit(&sleeptime,path)) {
+            syslog(L_ERROR, "%s can't check rate limit info", Client.host);
+            Reply("%d Retry later\r\n",
+                  ihave ? NNTP_FAIL_IHAVE_DEFER : NNTP_FAIL_POST_AUTH);
+            UnlockPostRec(path);
+            return;
+        } else if (sleeptime != 0L) {
+            syslog(L_NOTICE,"%s post sleep time is now %ld", Client.host, sleeptime);
+            sleep(sleeptime);
+        }
+      
+        /* Remove the lock here so that only one nnrpd process does the
+         * backoff sleep at once.  Other procs are sleeping for the lock. */
         UnlockPostRec(path);
-	return;
-      } else if (sleeptime != 0L) {
-        syslog(L_NOTICE,"%s post sleep time is now %ld", Client.host, sleeptime);
-        sleep(sleeptime);
-      }
-      
-      /* Remove the lock here so that only one nnrpd process does the
-       * backoff sleep at once.  Other procs are sleeping for the lock.
-       */
-      UnlockPostRec(path);
-
     } /* End backoff code. */
 
     /* Start at beginning of buffer. */
