@@ -1,7 +1,7 @@
 /*
 ** AUTHINFO SASL functionality.
 **
-** $Id: sasl.c 8523 2009-06-20 13:13:23Z iulius $
+** $Id: sasl.c 9871 2015-05-17 17:23:43Z iulius $
 */
 
 #include "config.h"
@@ -56,7 +56,7 @@ IsValidMechanism(const char *string)
 
     for (; *p != '\0'; p++) {
         len++;
-        if (!CTYPE(isalnum, *p) && *p != '-' && *p != '_')
+        if (!isalnum((unsigned char) *p) && *p != '-' && *p != '_')
             return false;
     }
 
@@ -117,6 +117,7 @@ SASLauth(int ac, char *av[])
     const char *mech;
     const char *clientin = NULL;
     unsigned int clientinlen = 0;
+    size_t tclientinlen = 0;
     const char *serverout = NULL;
     unsigned int serveroutlen;
     char base64[BASE64_BUF_SIZE+1];
@@ -172,8 +173,9 @@ SASLauth(int ac, char *av[])
             clientinlen = 0;
 	} else {
 	    /* Decode the response.  On error, SASL_CONTINUE should not be
-             * given.  Use SASL_BADPROT instead, in order to indicate
-             * a base64-encoding error. */
+             * given because we know for sure that we have already received
+             * the whole challenge/response.  Use SASL_BADPROT instead,
+             * in order to indicate a base64-encoding error. */
             r1 = sasl_decode64(clientin, strlen(clientin),
                                base64, BASE64_BUF_SIZE, &clientinlen);
             clientin = base64;
@@ -210,7 +212,8 @@ SASLauth(int ac, char *av[])
 
 	/* Get the response from the client. */
 	r1 = line_read(&NNTPline, PERMaccessconf->clienttimeout,
-		      &clientin, &clientinlen, NULL);
+		      &clientin, &tclientinlen, NULL);
+        clientinlen = tclientinlen;
         
         switch (r1) {
 	case RTok:
@@ -255,8 +258,9 @@ SASLauth(int ac, char *av[])
             clientinlen = 0;
         } else {
             /* Decode the response.  On error, SASL_CONTINUE should not be
-             * given.  Use SASL_BADPROT instead, in order to indicate
-             * a base64-encoding error. */
+             * given because we know for sure that we have already received
+             * the whole challenge/response.  Use SASL_BADPROT instead,
+             * in order to indicate a base64-encoding error. */
             r1 = sasl_decode64(clientin, clientinlen,
                                base64, BASE64_BUF_SIZE, &clientinlen);
             clientin = base64;
@@ -324,6 +328,9 @@ SASLauth(int ac, char *av[])
                 GRPcount = 0;
                 PERMgroupmadeinvalid = false;
             }
+
+            /* Reset our read buffer so as to prevent plaintext command injection. */
+            line_reset(&NNTPline);
         }
     } else {
 	/* Failure. */
