@@ -1,4 +1,4 @@
-/*  $Id: tls.c 8301 2009-01-29 18:50:30Z iulius $
+/*  $Id: tls.c 9785 2015-01-10 14:25:18Z iulius $
 **
 **  tls.c -- TLSv1 functions.
 **  Copyright (C) 2000 Kenichi Okada <okada@opaopa.org>.
@@ -378,16 +378,22 @@ set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 	if (key_file == NULL)
 	    key_file = cert_file;
 
-	/* Check ownership and permissions of key file. */
-	if (lstat(key_file, &buf) == -1) {
+	/* Check ownership and permissions of key file.
+         * Look at the real file (stat) and not a possible symlink (lstat). */
+	if (stat(key_file, &buf) == -1) {
 	    syslog(L_ERROR, "unable to stat private key '%s'", key_file);
 	    return (0);
 	}
-	if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0077) != 0 ||
-	    buf.st_uid != getuid()) {
+
+        /* Check that the key file is a real file, not readable by
+         * everyone.  If the mode is 440 or 640, make sure the group owner
+         * is the news group (to prevent the failure case of having news:users
+         * as the owner and group. */
+	if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0137) != 0
+            || ((buf.st_mode & 0040) != 0 && buf.st_gid != getegid())) {
 	    syslog(L_ERROR, "bad ownership or permissions on private key"
-                   " '%s': private key must be mode 600 and owned by "
-                   "uid %d", cert_file, getuid());
+                   " '%s':  private key must be mode 640 at most, and readable"
+                   " by the news group only", key_file);
 	    return (0);
 	}
 
@@ -407,6 +413,52 @@ set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 }
 
 
+#ifdef HAVE_OPENSSL_ECC
+/*
+**  Provide an ECKEY from a curve name.
+**  Accepts a NULL pointer as the name.
+**
+**  Returns the key, or NULL on error.
+*/
+static EC_KEY *
+eckey_from_name(char *name)
+{
+    EC_KEY *eckey;
+    size_t ncurves, nitems, i;
+    EC_builtin_curve *builtin_curves;
+    const char *sname;
+
+    if (name == NULL) {
+        return (NULL);
+    }
+
+    /* See EC_GROUP_new(3) for the details of this expressive dance. */
+    ncurves = EC_get_builtin_curves(NULL, 0); /* Number of curves. */
+
+    builtin_curves = xmalloc(ncurves * sizeof(EC_builtin_curve));
+    nitems = EC_get_builtin_curves(builtin_curves, ncurves);
+    if (nitems != ncurves) {
+        syslog(L_ERROR, "got %lu curves from EC_get_builtin_curves, "
+               "expected %lu", (unsigned long) nitems, (unsigned long) ncurves);
+    }
+    for (i = 0; i < nitems; i++) {
+        sname = OBJ_nid2sn(builtin_curves[i].nid);
+        if (strcmp(sname, name) == 0) {
+            break;
+        }
+    }
+    if (i == nitems) {
+        syslog(L_ERROR, "tlseccurve '%s' not found", name);
+        free(builtin_curves);
+        return (NULL);
+    }
+
+    eckey = EC_KEY_new_by_curve_name(builtin_curves[i].nid);
+    free(builtin_curves);
+    return (eckey);
+}
+#endif /* HAVE_OPENSSL_ECC */
+
 /*
 **  This is the setup routine for the SSL server.  As nnrpd might be called
 **  more than once, we only want to do the initialization one time.
@@ -419,7 +471,9 @@ set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 int
 tls_init_serverengine(int verifydepth, int askcert, int requirecert,
                       char *tls_CAfile, char *tls_CApath, char *tls_cert_file,
-                      char *tls_key_file)
+                      char *tls_key_file, bool prefer_server_ciphers,
+                      bool tls_compression, struct vector *tls_proto_vect,
+                      char *tls_ciphers, char *tls_ec_curve UNUSED)
 {
     int     off = 0;
     int     verify_flags = SSL_VERIFY_NONE;
@@ -428,6 +482,11 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
     char   *s_cert_file;
     char   *s_key_file;
     struct stat buf;
+    size_t  tls_protos = 0;
+    size_t  i;
+#ifdef HAVE_OPENSSL_ECC
+    EC_KEY *eckey;
+#endif
 
     if (tls_serverengine)
       return (0);				/* Already running. */
@@ -487,6 +546,93 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
     SSL_CTX_set_tmp_dh_callback(CTX, tmp_dh_cb);
     SSL_CTX_set_options(CTX, SSL_OP_SINGLE_DH_USE);
 
+#ifdef HAVE_OPENSSL_ECC
+    SSL_CTX_set_options(CTX, SSL_OP_SINGLE_ECDH_USE);
+
+    /* We set a curve here by name if provided
+     * or we use OpenSSL (>= 1.0.2) auto-selection
+     * or we default to NIST P-256. */
+    eckey = eckey_from_name(tls_ec_curve);
+    if (eckey != NULL) {
+        SSL_CTX_set_tmp_ecdh(CTX, eckey);
+    } else {
+# ifdef SSL_CTX_set_ecdh_auto
+        SSL_CTX_set_ecdh_auto(CTX, 1);
+# else
+        SSL_CTX_set_tmp_ecdh(CTX,
+                             EC_KEY_new_by_curve_name(NID_X9_62_prime256v1));
+# endif /* SSL_CTX_set_ecdh_auto */
+     }
+#endif /* HAVE_OPENSSL_ECC */
+
+    if (prefer_server_ciphers) {
+#ifdef SSL_OP_CIPHER_SERVER_PREFERENCE
+        SSL_CTX_set_options(CTX, SSL_OP_CIPHER_SERVER_PREFERENCE);
+#endif
+    }
+
+    if ((tls_proto_vect != NULL) && (tls_proto_vect->count > 0)) {
+        for (i = 0; i < tls_proto_vect->count; i++) {
+            if (tls_proto_vect->strings[i] != NULL) {
+                if (strcmp(tls_proto_vect->strings[i], "SSLv2") == 0) {
+                    tls_protos |= INN_TLS_SSLv2;
+                } else if (strcmp(tls_proto_vect->strings[i], "SSLv3") == 0) {
+                    tls_protos |= INN_TLS_SSLv3;
+                } else if (strcmp(tls_proto_vect->strings[i], "TLSv1") == 0) {
+                    tls_protos |= INN_TLS_TLSv1;
+                } else if (strcmp(tls_proto_vect->strings[i], "TLSv1.1") == 0) {
+                    tls_protos |= INN_TLS_TLSv1_1;
+                } else if (strcmp(tls_proto_vect->strings[i], "TLSv1.2") == 0) {
+                    tls_protos |= INN_TLS_TLSv1_2;
+                } else {
+                    syslog(L_ERROR, "TLS engine: unknown protocol '%s' in tlsprotocols",
+                           tls_proto_vect->strings[i]);
+                }
+            }
+        }
+    } else {
+        /* Default value:  allow all protocols. */
+        tls_protos = (INN_TLS_SSLv2 | INN_TLS_SSLv3 | INN_TLS_TLSv1
+                      | INN_TLS_TLSv1_1 | INN_TLS_TLSv1_2);
+    }
+
+    if ((tls_protos & INN_TLS_SSLv2) == 0) {
+        SSL_CTX_set_options(CTX, SSL_OP_NO_SSLv2);
+    }
+
+    if ((tls_protos & INN_TLS_SSLv3) == 0) {
+        SSL_CTX_set_options(CTX, SSL_OP_NO_SSLv3);
+    }
+
+    if ((tls_protos & INN_TLS_TLSv1) == 0) {
+        SSL_CTX_set_options(CTX, SSL_OP_NO_TLSv1);
+    }
+
+    if ((tls_protos & INN_TLS_TLSv1_1) == 0) {
+#ifdef SSL_OP_NO_TLSv1_1
+        SSL_CTX_set_options(CTX, SSL_OP_NO_TLSv1_1);
+#endif
+    }
+
+    if ((tls_protos & INN_TLS_TLSv1_2) == 0) {
+#ifdef SSL_OP_NO_TLSv1_2
+        SSL_CTX_set_options(CTX, SSL_OP_NO_TLSv1_2);
+#endif
+    }
+
+    if (tls_ciphers != NULL) {
+        if (SSL_CTX_set_cipher_list(CTX, tls_ciphers) == 0) {
+            syslog(L_ERROR, "TLS engine: cannot set cipher list");
+            return (-1);
+        }
+    }
+
+    if (!tls_compression) {
+#ifdef SSL_OP_NO_COMPRESSION
+        SSL_CTX_set_options(CTX, SSL_OP_NO_COMPRESSION);
+#endif
+    }
+
     verify_depth = verifydepth;
     if (askcert!=0)
 	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE;
@@ -504,7 +650,7 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
 
 /*
 **  The function called by nnrpd to initialize the TLS support.  Calls
-**  tls_init_server_engine and checks the result.  On any sort of failure,
+**  tls_init_serverengine and checks the result.  On any sort of failure,
 **  nnrpd will exit.
 **
 **  Returns -1 on error.
@@ -523,7 +669,13 @@ tls_init(void)
 				       innconf->tlscafile,
 				       innconf->tlscapath,
 				       innconf->tlscertfile,
-				       innconf->tlskeyfile);
+				       innconf->tlskeyfile,
+                                       innconf->tlspreferserverciphers,
+                                       innconf->tlscompression,
+                                       innconf->tlsprotocols,
+                                       innconf->tlsciphers,
+                                       innconf->tlseccurve);
+
     if (ssl_result == -1) {
         Reply("%d Error initializing TLS\r\n",
               initialSSL ? NNTP_FAIL_TERMINATING : NNTP_ERR_STARTTLS);
