@@ -1,9 +1,12 @@
+/*  $Id: buffindexed.h 9780 2015-01-07 21:03:00Z iulius $
+*/
+
 #ifndef _BUFFINDEXED_H_
 #define _BUFFINDEXED_H_
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+#include "config.h"
+
+BEGIN_DECLS
 
 bool buffindexed_open(int mode);
 bool buffindexed_groupstats(const char *group, int *lo, int *hi, int *count,
@@ -23,8 +26,6 @@ bool buffindexed_expiregroup(const char *group, int *lo, struct history *h);
 bool buffindexed_ctl(OVCTLTYPE type, void *val);
 void buffindexed_close(void);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+END_DECLS
 
 #endif /* _BUFFINDEXED_H_ */
