@@ -1,4 +1,4 @@
-/*  $Id: timecaf.c 8626 2009-09-07 09:06:42Z iulius $
+/*  $Id: timecaf.c 9871 2015-05-17 17:23:43Z iulius $
 **
 **  Like the timehash storage method (and heavily inspired by it), but uses
 **  the CAF library to store multiple articles in a single file.
@@ -97,17 +97,45 @@ typedef struct caftocl3cache CAFTOCL3CACHE;
 static CAFTOCL3CACHE *TOCCache[256]; /* indexed by storage class! */
 static int TOCCacheHits, TOCCacheMisses;
 
+/*
+**  The token is @04nn00aabbccyyyyxxxx0000000000000000@
+**  where "04" is the timecaf method number,
+**  "nn" the hexadecimal value of the storage class,
+**  "aabbccdd" the arrival time in hexadecimal (dd is unused),
+**  "xxxxyyyy" the hexadecimal sequence number seqnum.
+**
+**  innconf->patharticles + '/timecaf-nn/bb/aacc.CF'
+**  where "nn" is the hexadecimal value of the storage class,
+**  "aabbccdd" the arrival time in hexadecimal (dd is unused).
+*/
+char *
+timecaf_explaintoken(const TOKEN token)
+{
+    char                *text;
+    uint32_t            arrival;
+    uint16_t            seqnum1;
+    uint16_t            seqnum2;
+
+    memcpy(&arrival, &token.token[0], sizeof(arrival));
+    memcpy(&seqnum1, &token.token[4], sizeof(seqnum1));
+    memcpy(&seqnum2, &token.token[6], sizeof(seqnum2));
+ 
+    xasprintf(&text, "method=timecaf class=%u time=%lu seqnum=%lu file=%s/timecaf-%02x/%02x/%02x%02x.CF",
+              (unsigned int) token.class, ((unsigned long) (ntohl(arrival))) << 8,
+              ((unsigned long) ntohs(seqnum1)) + (((unsigned long) ntohs(seqnum2)) << 16),
+              innconf->patharticles, token.class,
+              (ntohl(arrival) >> 8) & 0xff,
+              (ntohl(arrival) >> 16) & 0xff,
+              ntohl(arrival) & 0xff);
+
+    return text;
+}
 
 static TOKEN MakeToken(time_t now, ARTNUM seqnum, STORAGECLASS class, TOKEN *oldtoken) {
     TOKEN               token;
     uint32_t            i;
     uint16_t            s;
 
-    /* The token is @04nn00aabbccyyyyxxxx0000000000000000@
-     * where "04" is the timecaf method number,
-     * "nn" the hexadecimal value of the storage class,
-     * "aabbccdd" the arrival time in hexadecimal (dd is unused),
-     * "xxxxyyyy" the hexadecimal sequence number seqnum. */
     if (oldtoken == (TOKEN *)NULL)
 	memset(&token, '\0', sizeof(token));
     else
@@ -144,9 +172,6 @@ static char *MakePath(time_t now, const STORAGECLASS class) {
     char *path;
     size_t length;
 
-    /* innconf->patharticles + '/timecaf-nn/bb/aacc.CF'
-     * where "nn" is the hexadecimal value of the storage class,
-     * "aabbccdd" the arrival time in hexadecimal (dd is unused). */
     length = strlen(innconf->patharticles) + 32;
     path = xmalloc(length);
     snprintf(path, length, "%s/timecaf-%02x/%02x/%02x%02x.CF",
@@ -160,15 +185,16 @@ static char *MakePath(time_t now, const STORAGECLASS class) {
 
 static TOKEN *PathNumToToken(char *path, ARTNUM artnum) {
     int			n;
-    unsigned int        t1, t2;
+    unsigned int        tclass, t1, t2;
     STORAGECLASS        class;
     time_t              timestamp;
     static TOKEN	token;
 
-    n = sscanf(path, "timecaf-%02x/%02x/%04x.CF", (unsigned int *)&class, &t1, &t2);
+    n = sscanf(path, "timecaf-%02x/%02x/%04x.CF", &tclass, &t1, &t2);
     if (n != 3)
 	return (TOKEN *)NULL;
     timestamp = ((t1 << 8) & 0xff00) | ((t2 << 8) & 0xff0000) | ((t2 << 0) & 0xff);
+    class = tclass;
     token = MakeToken(timestamp, artnum, class, (TOKEN *)NULL);
     return &token;
 }
@@ -687,22 +713,22 @@ static struct dirent *FindDir(DIR *dir, FINDTYPE type) {
         if (type == FIND_TOPDIR)
 	    if ((strlen(de->d_name) == 10) &&
 		(strncmp(de->d_name, "timecaf-", 8) == 0) &&
-		CTYPE(isxdigit, de->d_name[8]) &&
-		CTYPE(isxdigit, de->d_name[9]))
+		isxdigit((unsigned char) de->d_name[8]) &&
+		isxdigit((unsigned char) de->d_name[9]))
 	        return de;
 
 	if (type == FIND_DIR)
 	    if ((strlen(de->d_name) == 2)
-                && CTYPE(isxdigit, de->d_name[0])
-                && CTYPE(isxdigit, de->d_name[1]))
+                && isxdigit((unsigned char) de->d_name[0])
+                && isxdigit((unsigned char) de->d_name[1]))
 		return de;
 
 	if (type == FIND_CAF)
 	    if ((strlen(de->d_name) == 7) &&
-		CTYPE(isxdigit, de->d_name[0]) &&
-		CTYPE(isxdigit, de->d_name[1]) &&
-		CTYPE(isxdigit, de->d_name[2]) &&
-		CTYPE(isxdigit, de->d_name[3]) &&
+		isxdigit((unsigned char) de->d_name[0]) &&
+		isxdigit((unsigned char) de->d_name[1]) &&
+		isxdigit((unsigned char) de->d_name[2]) &&
+		isxdigit((unsigned char) de->d_name[3]) &&
 		(de->d_name[4] == '.') &&
 		(de->d_name[5] == 'C') &&
 		(de->d_name[6] == 'F'))
