@@ -1,4 +1,4 @@
-/*  $Id: tdx-group.c 7598 2007-02-09 02:40:51Z eagle $
+/*  $Id: tdx-group.c 9867 2015-05-14 13:44:14Z iulius $
 **
 **  Group index handling for the tradindexed overview method.
 **
@@ -96,8 +96,8 @@
 #include "inn/mmap.h"
 #include "inn/qio.h"
 #include "inn/vector.h"
-#include "libinn.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 #include "tdx-private.h"
 #include "tdx-structure.h"
 
@@ -278,7 +278,8 @@ file_open_group_index(struct group_index *index, struct stat *st)
 /*
 **  Given a group location, remap the index file if our existing mapping isn't
 **  large enough to include that group.  (This can be the case when another
-**  writer is appending entries to the group index.)
+**  writer is appending entries to the group index.)  Returns true on success
+**  (which includes "did not need to remap") and false on failure.
 */
 static bool
 index_maybe_remap(struct group_index *index, long loc)
@@ -357,7 +358,7 @@ index_expand(struct group_index *index)
        their next entry is entry 0.  We don't want to leave things in this
        state (particularly if this was the first expansion of the index file,
        in which case entry 0 points to entry 0 and our walking functions may
-       go into infinite loops.  Undo the file expansion. */
+       go into infinite loops).  Undo the file expansion. */
     if (!index_map(index)) {
         index->count -= 1024;
         if (ftruncate(index->fd, index_file_size(index->count)) < 0) {
@@ -380,7 +381,7 @@ index_expand(struct group_index *index)
         index->header->freelist.recno = i;
     }
 
-    inn_mapcntl(index->header, index_file_size(index->count), MS_ASYNC);
+    inn_msync_page(index->header, index_file_size(index->count), MS_ASYNC);
     return true;
 }
 
@@ -459,7 +460,7 @@ entry_splice(struct group_entry *entry, int *parent)
 {
     *parent = entry->next.recno;
     entry->next.recno = -1;
-    inn_mapcntl(parent, sizeof(*parent), MS_ASYNC);
+    inn_msync_page(parent, sizeof(*parent), MS_ASYNC);
 }
 
 
@@ -480,8 +481,8 @@ index_add(struct group_index *index, struct group_entry *entry)
     }
     entry->next.recno = index->header->hash[bucket].recno;
     index->header->hash[bucket].recno = entry_loc(index, entry);
-    inn_mapcntl(&index->header->hash[bucket], sizeof(struct loc), MS_ASYNC);
-    inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+    inn_msync_page(&index->header->hash[bucket], sizeof(struct loc), MS_ASYNC);
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
 }
 
 
@@ -502,11 +503,18 @@ index_find(struct group_index *index, const char *group)
 	return -1;
     loc = index->header->hash[index_bucket(hash)].recno;
 
-    while (loc >= 0 && loc < index->count) {
+    while (loc >= 0) {
         struct group_entry *entry;
 
-        if (loc > index->count && !index_maybe_remap(index, loc))
-            return -1;
+        if (loc >= index->count) {
+            if (!index_maybe_remap(index, loc)) {
+                return -1;
+            }
+            if (loc >= index->count) {
+                syswarn("tradindexed: entry %ld out of range", loc);
+                return -1;
+            }
+        }
         entry = index->entries + loc;
         if (entry->deleted == 0)
             if (memcmp(&hash, &entry->hash, sizeof(hash)) == 0)
@@ -529,8 +537,8 @@ freelist_add(struct group_index *index, struct group_entry *entry)
 {
     entry->next.recno = index->header->freelist.recno;
     index->header->freelist.recno = entry_loc(index, entry);
-    inn_mapcntl(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
-    inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+    inn_msync_page(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
 }
 
 
@@ -549,11 +557,20 @@ index_unlink_hash(struct group_index *index, HASH hash)
     parent = &index->header->hash[index_bucket(hash)].recno;
     current = *parent;
 
-    while (current >= 0 && current < index->count) {
+    while (current >= 0) {
         struct group_entry *entry;
 
-        if (current > index->count && !index_maybe_remap(index, current))
-            return -1;
+        if (current >= index->count) {
+            if (!index_maybe_remap(index, current)) {
+                return -1;
+            }
+            parent = &index->header->hash[index_bucket(hash)].recno;
+            current = *parent;
+            if (current < 0 || current >= index->count) {
+                syswarn("tradindexed: entry %ld out of range", current);
+                return -1;
+            }
+        }
         entry = &index->entries[current];
         if (entry->deleted == 0)
             if (memcmp(&hash, &entry->hash, sizeof(hash)) == 0) {
@@ -600,7 +617,7 @@ tdx_index_entry(struct group_index *index, const char *group)
         return NULL;
     entry = index->entries + loc;
     if (innconf->tradindexedmmap && innconf->nfsreader)
-	inn_mapcntl(entry, sizeof *entry, MS_INVALIDATE);
+	inn_msync_page(entry, sizeof *entry, MS_INVALIDATE);
     return entry;
 }
 
@@ -630,7 +647,7 @@ tdx_index_add(struct group_index *index, const char *group, ARTNUM low,
         entry = &index->entries[loc];
         if (entry->flag != *flag) {
             entry->flag = *flag;
-            inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         }
         return true;
     }
@@ -645,7 +662,7 @@ tdx_index_add(struct group_index *index, const char *group, ARTNUM low,
         }
     loc = index->header->freelist.recno;
     index->header->freelist.recno = index->entries[loc].next.recno;
-    inn_mapcntl(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
+    inn_msync_page(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
 
     /* Initialize the entry. */
     entry = &index->entries[loc];
@@ -820,11 +837,11 @@ tdx_data_add(struct group_index *index, struct group_entry *entry,
         old_base = entry->base;
         entry->indexinode = data->indexinode;
         entry->base = data->base;
-        inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+        inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         if (!tdx_data_pack_finish(data)) {
             entry->base = old_base;
             entry->indexinode = old_inode;
-            inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
             goto fail;
         }
     }
@@ -839,7 +856,14 @@ tdx_data_add(struct group_index *index, struct group_entry *entry,
     if (entry->high < article->number)
         entry->high = article->number;
     entry->count++;
-    inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+
+    /* Used to know that we have to remap the data file owing to our
+       OVSTATICSEARCH (an article whose number is lower than the highest has
+       been added at the end of the file). */
+    if (data->high > article->number)
+        data->remapoutoforder = true;
+
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
     index_lock_group(index->fd, offset, INN_LOCK_UNLOCK);
     return true;
 
@@ -880,7 +904,7 @@ tdx_index_rebuild_finish(struct group_index *index, struct group_entry *entry,
     *entry = *new;
     entry->indexinode = new_inode;
     new->indexinode = new_inode;
-    inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
     offset = entry - index->entries;
     index_lock_group(index->fd, offset, INN_LOCK_UNLOCK);
     return true;
@@ -930,12 +954,12 @@ tdx_expire(const char *group, ARTNUM *low, struct history *history)
     old_base = entry->base;
     entry->indexinode = new_entry.indexinode;
     entry->base = new_entry.base;
-    inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
     tdx_data_close(data);
     if (!tdx_data_rebuild_finish(group)) {
         entry->base = old_base;
         entry->indexinode = old_inode;
-        inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+        inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         goto fail;
     }
 
@@ -1043,7 +1067,7 @@ hashmap_load(void)
     struct hashmap *group;
     HASH grouphash;
 
-    activepath = concatpath(innconf->pathdb, _PATH_ACTIVE);
+    activepath = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
     active = QIOopen(activepath);
     free(activepath);
     if (active == NULL)
@@ -1070,7 +1094,8 @@ hashmap_load(void)
         hash_insert(hash, &group->hash, group);
         line = QIOread(active);
     }
-    cvector_free(data);
+    if (data != NULL)
+        cvector_free(data);
     QIOclose(active);
     return hash;
 }
@@ -1169,7 +1194,7 @@ index_audit_loc(struct group_index *index, int *loc, long number,
 
     if (fix && error) {
         *loc = -1;
-        inn_mapcntl(loc, sizeof(*loc), MS_ASYNC);
+        inn_msync_page(loc, sizeof(*loc), MS_ASYNC);
     }
 }
 
@@ -1187,7 +1212,7 @@ index_audit_deleted(struct group_entry *entry, long number, bool fix)
              number);
         if (fix) {
             HashClear(&entry->hash);
-            inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         }
     }
 }
@@ -1327,7 +1352,7 @@ index_audit_group(struct group_index *index, struct group_entry *entry,
     } else {
         if (entry->flag != group->flag) {
             entry->flag = group->flag;
-            inn_mapcntl(entry, sizeof(*entry), MS_ASYNC);
+            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         }
         tdx_data_audit(group->name, entry, fix);
     }
@@ -1410,6 +1435,10 @@ tdx_index_audit(bool fix)
        need to map hashes to group names, so load a hash of the active file to
        do that resolution. */
     hashmap = hashmap_load();
+    if (hashmap == NULL) {
+        warn("tradindexed: cannot hash active file");
+        return;
+    }
     data.index = index;
     data.fix = fix;
     hash_traverse(hashmap, index_audit_active, &data);
@@ -1419,6 +1448,5 @@ tdx_index_audit(bool fix)
             continue;
         index_audit_group(index, entry, hashmap, fix);
     }
-    if (hashmap != NULL)
-        hash_free(hashmap);
+    hash_free(hashmap);
 }
