@@ -1,4 +1,4 @@
-/* $Id: artparse-t.c 8409 2009-04-11 22:13:41Z iulius $ */
+/* $Id: artparse-t.c 9140 2010-10-29 18:24:47Z iulius $ */
 /* Test suite for ARTparse. */
 
 #include "config.h"
@@ -29,17 +29,15 @@ const struct {
     { "../data/articles/6",
       "437 Article of 8193 bytes exceeds local limit of 8192 bytes" },
     { "../data/articles/bad-empty",
-      "437 Empty article" },
+      "437 Empty headers and body" },
     { "../data/articles/bad-hdr-nospc",
       "437 No colon-space in \"Test:<-he: re\" header" },
     { "../data/articles/bad-hdr-space",
       "437 Space before colon in \"Test\" header" },
     { "../data/articles/bad-hdr-trunc",
       "437 No colon-space in \"Test:\" header" },
-    { "../data/articles/bad-long-cont",
-      "437 Header line too long (1025 bytes)" },
-    { "../data/articles/bad-long-hdr",
-      "437 Header line too long (1025 bytes)" },
+    { "../data/articles/bad-long-cont", "" }, /* Not an error for a relaying agent. */
+    { "../data/articles/bad-long-hdr", "" }, /* Not an error for a relaying agent. */
     { "../data/articles/bad-no-body",
       "437 No body" },
     { "../data/articles/bad-no-header",
@@ -107,7 +105,7 @@ initialize(void)
    (which may be ""), create a channel, run the article through ARTparse
    either all at once or, if slow is true, one character at a time, and check
    the result.  If shift is true, shift the start of the article in the buffer
-   by a random amount.  Produces three test results. */
+   by a random amount.  Produces five test results. */
 static void
 ok_article(int n, const char *path, const char *error, bool slow, bool shift)
 {
@@ -151,7 +149,7 @@ ok_article(int n, const char *path, const char *error, bool slow, bool shift)
         message_handlers_warn(1, message_log_stderr);
     }
     ok(n++, okay);
-    if (wirelen > (size_t) innconf->maxartsize)
+    if (wirelen > innconf->maxartsize)
         expected = CSgotlargearticle;
     else if (wirelen == 5)
         expected = CSnoarticle;
