@@ -1,11 +1,11 @@
-/* $Id: md5-t.c 9827 2015-04-23 14:50:38Z iulius $ */
+/* $Id: md5-t.c 9828 2015-04-23 14:51:16Z iulius $ */
 /* MD5 hashing test suite. */
 
 #include "config.h"
 #include "clibrary.h"
 #include "inn/md5.h"
 #include "inn/libinn.h"
-#include "tap/basic.h"
+#include "libtest.h"
 
 /* Used for strings of unsigned characters (called SUC instead of U
    because it otherwise conflicts with Unicode strings). */
