@@ -1,27 +1,24 @@
-/* $Id: qio-t.c 10457 2020-12-19 06:10:49Z eagle $ */
+/* $Id: qio-t.c 8407 2009-04-11 17:16:06Z iulius $ */
 /* Test suite for the Quick I/O library */
 
-#define LIBTEST_NEW_FORMAT 1
-
 #include "config.h"
 #include "clibrary.h"
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 
+#include "inn/messages.h"
 #include "inn/qio.h"
 #include "inn/libinn.h"
-#include "tap/basic.h"
-
+#include "libtest.h"
 
 static void
 output(int fd, const void *data, size_t size)
 {
     if (xwrite(fd, data, size) < 0)
-        sysbail("Can't write to .testout");
+        sysdie("Can't write to .testout");
 }
 
-
 int
 main(void)
 {
@@ -29,9 +26,14 @@ main(void)
     unsigned char c;
     char *result;
     int i, count, fd;
+    size_t size = 8192;
     QIOSTATE *qio;
     bool success;
 
+#if HAVE_STRUCT_STAT_ST_BLKSIZE
+    struct stat st;
+#endif
+
     for (c = 1, i = 0; i < 255; i++, c++)
         data[i] = c;
     data[9] = ' ';
@@ -41,15 +43,27 @@ main(void)
     memcpy(out, data, 255);
     out[255] = '\0';
     fd = open(".testout", O_RDWR | O_CREAT | O_TRUNC, 0644);
-    if (fd < 0)
-        sysbail("Can't create .testout");
+    if (fd < 0) sysdie("Can't create .testout");
+
+#if HAVE_STRUCT_STAT_ST_BLKSIZE
+    /* Mostly duplicate the code from qio.c so that we can test with lines
+       exactly as large as the buffer. */
+    if (fstat(fd, &st) == 0 && S_ISREG(st.st_mode)) {
+        size = st.st_blksize;
+        if (size > 4 * 8192)
+            size = 8192;
+	else
+	    while(size < 8192)
+		size += st.st_blksize;
+    }
+#endif /* HAVE_STRUCT_STAT_ST_BLKSIZE */
 
     /* Start with small, equally sized lines exactly equal to the buffer.
        Then a line equal in size to the buffer, then a short line and
        another line equal in size to the buffer, then a half line and lines
        repeated to fill another buffer, then a line that's one character too
        long.  Finally, put out another short line. */
-    count = QIO_BUFFERSIZE / 256;
+    count = size / 256;
     for (i = 0; i < count; i++)
         output(fd, line, 256);
     for (i = 0; i < count - 1; i++)
@@ -69,84 +83,82 @@ main(void)
     output(fd, line, 256);
     close(fd);
 
-    plan(36);
+    test_init(36);
 
     /* Now make sure we can read all that back correctly. */
     qio = QIOopen(".testout");
-    ok(qio != NULL, "QIOpen works");
-    ok(!QIOerror(qio), "...and there is no error state");
-    ok(QIOfileno(qio) > 0, "...and QIOfileno returns something valid");
+    ok(1, qio != NULL);
+    ok(2, !QIOerror(qio));
+    ok(3, QIOfileno(qio) > 0);
     if (unlink(".testout") < 0)
-        sysbail("Can't unlink .testout");
+        sysdie("Can't unlink .testout");
     for (success = true, i = 0; i < count; i++) {
         result = QIOread(qio);
         success = (success && !QIOerror(qio) && (QIOlength(qio) == 255)
                    && !strcmp(result, (char *) out));
     }
-    ok(success, "Able to read the first QIO_BUFFERSIZE of data");
-    is_int(QIOtell(qio), (off_t) QIO_BUFFERSIZE, "QIOtell is correct");
+    ok(4, success);
+    ok(5, QIOtell(qio) == (off_t) size);
     result = QIOread(qio);
-    if (strlen(result) < QIO_BUFFERSIZE - 1) {
-        ok(false, "Read largest possible line");
+    if (strlen(result) < size - 1) {
+        ok(6, false);
     } else {
         for (success = true, i = 0; i < count - 1; i++)
             success = success && !memcmp(result + i * 256, data, 256);
         success = success && !memcmp(result + i * 256, data, 255);
-        ok(success, "Read largest possible line");
+        ok(6, success);
     }
-    is_int(QIOtell(qio), (off_t) (2 * QIO_BUFFERSIZE), "QIOtell is correct");
+    ok(7, QIOtell(qio) == (off_t) (2 * size));
     result = QIOread(qio);
-    ok(!QIOerror(qio), "No error on reading an empty line");
-    is_int(QIOlength(qio), 0, "Length of the line is 0");
-    is_string(result, "", "...and result is an empty string");
+    ok(8, !QIOerror(qio));
+    ok(9, QIOlength(qio) == 0);
+    ok(10, *result == 0);
     result = QIOread(qio);
-    if (strlen(result) < QIO_BUFFERSIZE - 1) {
-        ok(false, "Read largest line again");
+    if (strlen(result) < size - 1) {
+        ok(11, false);
     } else {
         for (success = true, i = 0; i < count - 1; i++)
             success = success && !memcmp(result + i * 256, data, 256);
         success = success && !memcmp(result + i * 256, data, 255);
-        ok(success, "Read largest line again");
+        ok(11, success);
     }
-    is_int(QIOtell(qio), (off_t) (3 * QIO_BUFFERSIZE + 1),
-           "QIOtell is correct");
+    ok(12, QIOtell(qio) == (off_t) (3 * size + 1));
     result = QIOread(qio);
-    ok(!QIOerror(qio), "No error on a shorter read");
-    is_int(QIOlength(qio), 127, "Length is 127");
-    is_int(strlen(result), 127, "String is 127");
-    ok(!memcmp(result, data, 127), "Data is correct");
+    ok(13, !QIOerror(qio));
+    ok(14, QIOlength(qio) == 127);
+    ok(15, strlen(result) == 127);
+    ok(16, !memcmp(result, data, 127));
     for (success = true, i = 0; i < count; i++) {
         result = QIOread(qio);
         success = (success && !QIOerror(qio) && (QIOlength(qio) == 255)
                    && !strcmp(result, (char *) out));
     }
-    ok(success, "Able to read another batch of lines");
-    is_int(QIOtell(qio), (off_t) (4 * QIO_BUFFERSIZE + 129),
-           "QIOtell is correct");
+    ok(17, success);
+    ok(18, QIOtell(qio) == (off_t) (4 * size + 129));
     result = QIOread(qio);
-    ok(!result, "Failed to read too long of line");
-    ok(QIOerror(qio), "Error reported");
-    ok(QIOtoolong(qio), "...and too long flag is set");
+    ok(19, !result);
+    ok(20, QIOerror(qio));
+    ok(21, QIOtoolong(qio));
     result = QIOread(qio);
-    ok(!QIOerror(qio), "Reading again succeeds");
-    is_int(QIOlength(qio), 255, "...and returns the next block");
-    is_int(strlen(result), 255, "...and length is correct");
-    ok(!memcmp(result, line, 255), "...and data is correct");
+    ok(22, !QIOerror(qio));
+    ok(23, QIOlength(qio) == 255);
+    ok(24, strlen(result) == 255);
+    ok(25, !memcmp(result, line, 255));
     result = QIOread(qio);
-    ok(!result, "End of file reached");
-    ok(!QIOerror(qio), "...with no error");
-    is_int(QIOrewind(qio), 0, "QIOrewind works");
-    is_int(QIOtell(qio), 0, "...and QIOtell is correct");
+    ok(26, !result);
+    ok(27, !QIOerror(qio));
+    ok(28, QIOrewind(qio) == 0);
+    ok(29, QIOtell(qio) == 0);
     result = QIOread(qio);
-    ok(!QIOerror(qio), "Reading the first line works");
-    is_int(QIOlength(qio), 255, "...and QIOlength is correct");
-    is_int(strlen(result), 255, "...and the length is correct");
-    ok(!strcmp(result, (char *) out), "...and the data is correct");
-    is_int(QIOtell(qio), 256, "...and QIOtell is correct");
+    ok(30, !QIOerror(qio));
+    ok(31, QIOlength(qio) == 255);
+    ok(32, strlen(result) == 255);
+    ok(33, !strcmp(result, (char *) out));
+    ok(34, QIOtell(qio) == 256);
     fd = QIOfileno(qio);
     QIOclose(qio);
-    ok(close(fd) < 0, "QIOclose closed the file descriptor");
-    is_int(errno, EBADF, "...as confirmed by errno");
+    ok(35, close(fd) < 0);
+    ok(36, errno == EBADF);
 
     return 0;
 }
