@@ -1,4 +1,4 @@
-/* $Id: tst-t.c 10015 2016-05-05 12:43:09Z iulius $ */
+/* $Id: tst-t.c 9828 2015-04-23 14:51:16Z iulius $ */
 /* Test suite for ternary search tries. */
 
 #include "config.h"
@@ -7,7 +7,7 @@
 #include "inn/messages.h"
 #include "inn/tst.h"
 #include "inn/libinn.h"
-#include "tap/basic.h"
+#include "libtest.h"
 
 /* Used for strings of unsigned characters (called SUC instead of U
    because it otherwise conflicts with Unicode strings). */
@@ -31,7 +31,7 @@ main(void)
     char foo[] = "foo";
     char testing[] = "testing";
     char Strange[] = "Strange";
-    char change[] = "\231hange";
+    char change[] = "hange";
 
     test_init(38);
 
@@ -47,13 +47,13 @@ main(void)
     ok(9, tst_insert(tst, SUC"testing", testing, 0, NULL) == TST_OK);
     ok(10, tst_insert(tst, SUC"t", t, 0, NULL) == TST_OK);
     ok(11, tst_insert(tst, SUC"Strange", Strange, 0, NULL) == TST_OK);
-    ok(12, tst_insert(tst, SUC"\231hange", change, 0, NULL) == TST_OK);
+    ok(12, tst_insert(tst, SUC"hange", change, 0, NULL) == TST_OK);
     ok(13, tst_insert(tst, SUC"", foo, 0, NULL) == TST_NULL_KEY);
     ok(14, tst_insert(tst, NULL, foo, 0, NULL) == TST_NULL_KEY);
     ok_string(15, "testing", tst_search(tst, SUC"testing"));
     ok_string(16, "t", tst_search(tst, SUC"t"));
     ok_string(17, "Strange", tst_search(tst, SUC"Strange"));
-    ok_string(18, "\231hange", tst_search(tst, SUC"\231hange"));
+    ok_string(18, "hange", tst_search(tst, SUC"hange"));
     ok_string(19, "foo", tst_search(tst, SUC"test"));
     ok(20, tst_search(tst, SUC"") == NULL);
     ok(21, tst_search(tst, SUC"Peter") == NULL);
@@ -67,7 +67,7 @@ main(void)
     ok_string(29, "foo", tst_search(tst, SUC"test"));
     ok_string(30, "testing", tst_delete(tst, SUC"testing"));
     ok_string(31, "foo", tst_search(tst, SUC"test"));
-    ok_string(32, "\231hange", tst_delete(tst, SUC"\231hange"));
+    ok_string(32, "hange", tst_delete(tst, SUC"hange"));
     ok_string(33, "foo", tst_delete(tst, SUC"test"));
     ok(34, tst_search(tst, NULL) == NULL);
     ok(35, tst_delete(tst, NULL) == NULL);
@@ -102,10 +102,8 @@ main(void)
     }
     ok(37, !reported);
 
-    if (fseek(words, 0, SEEK_SET) < 0) {
-        fclose(words);
+    if (fseek(words, 0, SEEK_SET) < 0)
         sysdie("Unable to rewind words file");
-    }
     reported = false;
     if (tst == NULL)
         reported = true;
@@ -132,7 +130,6 @@ main(void)
         }
     }
     tst_cleanup(tst);
-    fclose(words);
     ok(38, !reported);
 
     return 0;
